@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-dodgy-imports #-} -- needed for GHC 7.0/7.2
 {-# LANGUAGE BangPatterns, ForeignFunctionInterface, CPP, ScopedTypeVariables #-}
 
 -----------------------------------------------------------------------------
@@ -70,12 +71,12 @@ import Control.Exception.Extensible ( catch, SomeException )
 #endif
 import System.IO
 import System.IO.Unsafe         ( unsafePerformIO )
-import System.Console.Haskeline ( InputT, runInputTBehavior, defaultSettings, useFileHandle )
-import System.Console.Haskeline.Encoding ( decode, encode )
+
+import Encoding ( decode, encode )
 
 import Foreign.Storable         ( peekElemOff, peek )
 import Foreign.Marshal.Array    ( advancePtr )
-import Foreign.C.Types          ( CInt )
+import Foreign.C.Types          ( CInt(..) )
 
 import Data.Bits                ( rotateL )
 import Data.Char                ( ord, isSpace )
@@ -517,11 +518,6 @@ breakBeforeNthNewline n the_ps =
           end = the_s + l
       findit n the_s
 
--- we use a custom Behavior because otherwise on Windows haskeline tries to
--- do hGetEcho on stdin which fails if stdin is e.g. /dev/null
-unsafeRunInput :: InputT IO a -> a
-unsafeRunInput = unsafePerformIO . runInputTBehavior (useFileHandle stdin) defaultSettings
-
 -- | Test if a ByteString is made of ascii characters
 isAscii :: B.ByteString -> Bool
 isAscii = B.all (\w -> w < 128)
@@ -531,7 +527,7 @@ isAscii = B.all (\w -> w < 128)
 -- and above also supply locale conversion with functions with a pure type.
 -- Unrecognized byte sequences in the input are skipped.
 decodeLocale :: B.ByteString -> String
-decodeLocale = unsafeRunInput . decode
+decodeLocale = unsafePerformIO . decode
 
 -- | Encode a String to a ByteString with latin1 (i.e., the values of the
 -- characters become the values of the bytes; if a character value is greater
@@ -541,7 +537,7 @@ encodeLatin1 = B.pack . (map (fromIntegral . ord))
 
 -- | Encode a String to a ByteString according to the current locale
 encodeLocale :: String -> B.ByteString
-encodeLocale = unsafeRunInput . encode
+encodeLocale = unsafePerformIO . encode
 
 -- | Take a @String@ that represents byte values and re-decode it acording to
 -- the current locale.
