@@ -12,7 +12,7 @@ _bug bs s = _bugDoc bs (text s)
 _bugDoc :: BugStuff -> Doc -> a
 _bugDoc bs s = errorDoc $
    text ("bug at " ++ _bugLoc bs) $$ s $$
-   text ("See http://wiki.darcs.net/index.html/BugTrackerHowto " ++
+   text ("See http://wiki.darcs.net/BugTracker/Reporting " ++
          "for help on bug reporting.")
 
 _bugLoc :: BugStuff -> String
