@@ -1,6 +1,8 @@
 {-# OPTIONS -fno-warn-orphans #-}
 module Darcs.ColorPrinter ( errorDoc, traceDoc, assertDoc, fancyPrinters ) where
 
+import Prelude hiding ( catch )
+import Control.Exception ( catch, IOException )
 import Debug.Trace ( trace )
 import System.IO ( stderr )
 import Darcs.External (getTermNColors)
@@ -93,8 +95,8 @@ getPolicy handle = unsafePerformIO $
                   }
  where
   getEnvBool s = safeGetEnv s >>= return.(/= "0")
-  safeGetEnv s = getEnv s `catch` \_ -> return "0"
-  getEnvString s = getEnv s `catch` \_ -> return ""
+  safeGetEnv s = getEnv s `catch` \(_ :: IOException) -> return "0"
+  getEnvString s = getEnv s `catch` \(_ :: IOException) -> return ""
 
 
 -- printers
