@@ -20,12 +20,13 @@
 #include "gadts.h"
 
 module Darcs.Commands.Changes ( changes, log ) where
-import Prelude hiding ( log )
+import Prelude hiding ( log, catch )
 import Unsafe.Coerce (unsafeCoerce)
 
 import Data.List ( intersect, sort, nub, find )
 import Data.Maybe ( fromMaybe, fromJust, isJust )
 import Control.Arrow ( second )
+import Control.Exception ( catch, IOException )
 import Control.Monad.State.Strict
 import Control.Applicative ((<$>))
 
@@ -126,7 +127,7 @@ showChanges opts files =
   Sealed unrec <- case files of
     Nothing -> return $ Sealed NilFL
     Just _ -> Sealed `fmap` unrecordedChanges (UseIndex, ScanKnown) repository files
-                  `catch` \_ -> return (Sealed NilFL) -- this is triggered when repository is remote
+                  `catch` \(_ :: IOException) -> return (Sealed NilFL) -- this is triggered when repository is remote
   let normfp = fn2fp . normPath . fp2fn
       undoUnrecordedOnFPs = effectOnFilePaths (invert unrec)
       recFiles = map normfp . undoUnrecordedOnFPs . map toFilePath <$> files
