@@ -18,12 +18,16 @@
 {-# LANGUAGE CPP, TypeOperators #-}
 
 module Darcs.Commands.Send ( send ) where
+
+import Prelude hiding ( catch )
+
 import System.Exit ( exitWith, ExitCode( ExitSuccess ) )
 #ifndef HAVE_MAPI
 import System.Exit ( ExitCode( ExitFailure ) )
 #endif
 import System.IO.Error ( ioeGetErrorString )
 import System.IO ( hClose )
+import Control.Exception ( catch, IOException )
 import Control.Monad ( when, unless, forM_ )
 import Storage.Hashed.Tree ( Tree )
 import Data.List ( intercalate, isPrefixOf, stripPrefix )
@@ -282,7 +286,7 @@ sendBundle opts to_be_sent bundle fname wtds their_name=
              (\url -> do
                 putInfo opts . text $ "Posting patch to " ++ url
                 postUrl url (BC.unpack nbody) "message/rfc822")
-             `catch` const sendmail
+             `catch` (\(_ :: IOException) -> sendmail)
            cleanup opts mailfile
 
 generateEmailToString :: [WhatToDo] -> String
