@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-dodgy-imports #-} -- needed for GHC 7.0/7.2
 {-# LANGUAGE CPP, ForeignFunctionInterface #-}
 
 module Darcs.Compat (stdoutIsAPipe, mkStdoutTemp, canonFilename,
@@ -15,7 +16,7 @@ import Foreign.C.String ( peekCString )
 #endif
 
 import Control.Monad ( unless )
-import Foreign.C.Types ( CInt )
+import Foreign.C.Types ( CInt(..) )
 import Foreign.C.String ( CString, withCString )
 import Foreign.C.Error ( throwErrno, eEXIST, getErrno )
 import System.Directory ( getCurrentDirectory )
