@@ -82,7 +82,7 @@ withLock s job = bracket (getlock s 30) releaseLock (\_ -> job)
 withLockCanFail :: String -> IO a -> IO (Either () a)
 withLockCanFail s job =
   bracket (takeLock s)
-          (\l -> if l then releaseLock s else return ())
+          (\l -> when l $ releaseLock s)
           (\l -> if l then job >>= (return.Right)
                       else return $ Left ())
 
@@ -141,14 +141,15 @@ withTemp = bracket get_empty_file removeFileMayNotExist
                               hClose h
                               return f
 
--- |'withOpenTemp' creates an already open temporary
--- file.  Both of them run their argument and then delete the file.  Also,
+-- |'withOpenTemp' creates a temporary file, and opens it.
+-- Both of them run their argument and then delete the file.  Also,
 -- both of them (to my knowledge) are not susceptible to race conditions on
 -- the temporary file (as long as you never delete the temporary file; that
 -- would reintroduce a race condition).
 withOpenTemp :: ((Handle, String) -> IO a) -> IO a
 withOpenTemp = bracket get_empty_file cleanup
-    where cleanup (h,f) = do try (hClose h) :: IO (Either SomeException ())
+-- Warning:  A do-notation statement discarded a result of type Either SomeException ().
+    where cleanup (h,f) = do _ <- try (hClose h) :: IO (Either SomeException ())
                              removeFileMayNotExist f
           get_empty_file = invert `fmap` openBinaryTempFile "." "darcs"
           invert (a,b) = (b,a)
@@ -254,10 +255,10 @@ rmRecursive d =
     do isd <- doesDirectoryReallyExist d
        if not isd
           then removeFile d
-          else when isd $ do conts <- actual_dir_contents
-                             withCurrentDirectory d $
-                               mapM_ rmRecursive conts
-                             removeDirectory d
+          else do conts <- actual_dir_contents
+                  withCurrentDirectory d $
+                    mapM_ rmRecursive conts
+                  removeDirectory d
     where actual_dir_contents = -- doesn't include . or ..
               do c <- getDirectoryContents d
                  return $ filter (/=".") $ filter (/="..") c
@@ -278,7 +279,8 @@ withNamedTemp n = bracket get_empty_file removeFileMayNotExist
 editText :: String -> B.ByteString -> IO B.ByteString
 editText desc txt = withNamedTemp desc $ \f -> do
   B.writeFile f txt
-  runEditor f
+-- Warning:  A do-notation statement discarded a result of type ExitCode.
+  _ <- runEditor f
   B.readFile f
 
 readBinFile :: FilePathLike p => p -> IO String
