@@ -15,9 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP #-}
--- , TypeOperators, GeneralizedNewtypeDeriving, ScopedTypeVariables #-}
 
 #include "gadts.h"
 
@@ -63,15 +61,22 @@ module Darcs.Patch.Choices ( PatchChoices, patchChoices, patchChoicesTps,
                     ) where
 
 import Control.Monad.Identity ( Identity )
-import Control.Monad.State( StateT(..) )
+import Control.Monad.State ( StateT(..) )
+
+import Prelude hiding ( pred )
 
 import Darcs.Patch
+     ( Patchy, commuteRL, commute, merge, listTouchedFiles, hunkMatches
+     , invert )
+import Darcs.Patch.Merge ( Merge )
 import Darcs.Patch.Permutations ( commuteWhatWeCanRL, commuteWhatWeCanFL )
-import Darcs.Patch.Patchy ( Invert, Commute )
-import Darcs.Witnesses.Ordered ( FL(..), RL(..), MyEq, unsafeCompare, EqCheck(..),
+import Darcs.Patch.Patchy ( Invert, Commute, PatchInspect )
+import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Witnesses.Ordered ( FL(..), RL(..),
                              (:>)(..), (:\/:)(..), (:/\:)(..), (:||:)(..),
                              zipWithFL, mapFL_FL, concatFL,
-                             (+>+), reverseRL, unsafeCoerceP, anyFL )
+                             (+>+), reverseRL, anyFL )
 import Darcs.Witnesses.Sealed ( Sealed2(..) )
 
 
@@ -96,11 +101,11 @@ data TaggedPatch p C(x y) = TP Tag (p C(x y))
 -- | The @Bool@ parameter indicates whether the patch has been explicitely
 -- selected (or rejected) by the user.
 data PatchChoice p C(x y) = PC { pcPatch :: (TaggedPatch p C(x y))
-                               , choice :: Bool}
+                               , _pcChoice :: Bool}
 
 data PatchChoices p C(x y) where
-  PCs { firsts :: FL (TaggedPatch p) C(x m)
-      , lasts :: FL (PatchChoice p) C(m y)} :: PatchChoices p C(x y)
+  PCs { pcsFirsts :: FL (TaggedPatch p) C(x m)
+      , pcsLasts :: FL (PatchChoice p) C(m y)} :: PatchChoices p C(x y)
 
 -- | See module documentation for 'Darcs.Patch.Choices'
 data Slot = InFirst | InMiddle | InLast
@@ -125,21 +130,28 @@ instance MyEq p => MyEq (TaggedPatch p) where
 
 instance Invert p => Invert (TaggedPatch p) where
     invert = liftTP invert
-    identity = TP (TG Nothing (-1)) identity
 
 instance Commute p => Commute (TaggedPatch p) where
     commute (TP t1 p1 :> TP t2 p2) = do p2' :> p1' <- commute (p1 :> p2)
                                         return (TP t2 p2' :> TP t1 p1')
+
+instance PatchInspect p => PatchInspect (TaggedPatch p) where
     listTouchedFiles (TP _ p) = listTouchedFiles p
     hunkMatches f (TP _ p) = hunkMatches f p
+
+instance Merge p => Merge (TaggedPatch p) where
     merge (TP t1 p1 :\/: TP t2 p2) = case merge (p1 :\/: p2) of
                                      p2' :/\: p1' -> TP t2 p2' :/\: TP t1 p1'
 
 instance Commute p => Commute (PatchChoice p) where
   commute (PC p1 c1 :> PC p2 c2) = do p2' :> p1' <- commute (p1 :> p2)
                                       return (PC p2' c2 :> PC p1' c1)
+
+instance PatchInspect p => PatchInspect (PatchChoice p) where
   listTouchedFiles (PC p _) = listTouchedFiles p
   hunkMatches f (PC p _) = hunkMatches f p
+
+instance Merge p => Merge (PatchChoice p) where
   merge (PC tp1 c1 :\/: PC tp2 c2) = case merge (tp1 :\/: tp2) of
     tp2' :/\: tp1' -> PC tp2' c2 :/\: PC tp1' c1
 
@@ -215,33 +227,33 @@ patchSlot (TP t _) pc@(PCs f l) =
   else psLast f NilRL NilRL l
   where
     foundIn = anyFL ((== t) . tag)
-    psLast :: FORALL(m b l)
+    psLast :: FORALL(m c l)
              FL (TaggedPatch p) C(x m) ->
-             RL (TaggedPatch p) C(m b) ->
-             RL (TaggedPatch p) C(b l) ->
+             RL (TaggedPatch p) C(m c) ->
+             RL (TaggedPatch p) C(c l) ->
              FL (PatchChoice p) C(l y) ->
              (Slot, PatchChoices p C(x y))
     psLast firsts middles bubble (PC tp True :>: ls)
       | tag tp == t = (InLast
-                      , PCs { firsts = firsts
-                            , lasts = settleM middles
-                                      +>+ settleB bubble
-                                      +>+ PC tp True :>: ls})
+                      , PCs { pcsFirsts = firsts
+                            , pcsLasts = settleM middles
+                                         +>+ settleB bubble
+                                         +>+ PC tp True :>: ls})
     psLast firsts middles bubble (PC tp False :>: ls)
       | tag tp == t =
         case commuteRL (bubble :> tp) of
         Just (tp' :> bubble') -> (InMiddle,
-                                 PCs { firsts = firsts
-                                     , lasts = settleM middles
-                                               +>+ PC tp' False
-                                               :>: settleB bubble'
-                                               +>+ ls})
+                                 PCs { pcsFirsts = firsts
+                                     , pcsLasts = settleM middles
+                                                  +>+ PC tp' False
+                                                  :>: settleB bubble'
+                                                  +>+ ls})
         Nothing -> (InLast,
-                   PCs { firsts = firsts
-                       , lasts = settleM middles
-                                 +>+ settleB bubble
-                                 +>+ PC tp True
-                                 :>: ls})
+                   PCs { pcsFirsts = firsts
+                       , pcsLasts = settleM middles
+                                    +>+ settleB bubble
+                                    +>+ PC tp True
+                                    :>: ls})
     psLast firsts middles bubble (PC tp True :>: ls) =
       psLast firsts middles (tp :<: bubble) ls
     psLast firsts middles bubble (PC tp False :>: ls) =
@@ -260,8 +272,8 @@ forceMatchingFirst :: forall p C(a b). Patchy p =>
                       ( FORALL(x y) TaggedPatch p C(x y) -> Bool)
                       -> PatchChoices p C(a b)
                       -> PatchChoices p C(a b)
-forceMatchingFirst pred (PCs f l) =
-  fmfLasts f NilRL l
+forceMatchingFirst pred (PCs fn l) =
+  fmfLasts fn NilRL l
     where
       fmfLasts :: FL (TaggedPatch p) C(a m)
                  -> RL (PatchChoice p) C(m n)
@@ -275,8 +287,8 @@ forceMatchingFirst pred (PCs f l) =
                   f' = f +>+ mapFL_FL pcPatch (reverseRL deps) +>+ (pcPatch a' :>: NilFL)
                 in fmfLasts f' l1' l2
       fmfLasts f l1 (a :>: l2) = fmfLasts f (a :<: l1) l2
-      fmfLasts f l1 NilFL = PCs { firsts = f
-                                , lasts = reverseRL l1 }
+      fmfLasts f l1 NilFL = PCs { pcsFirsts = f
+                                , pcsLasts = reverseRL l1 }
       pred_pc :: FORALL(x y) PatchChoice p C(x y) -> Bool
       pred_pc (PC tp _) = pred tp
 
@@ -329,8 +341,8 @@ fmlFirst pred b f1 (a :>: f2) l
               in
               fmlFirst pred b f1 f2' l'
 fmlFirst pred b f1 (a :>: f2) l = fmlFirst pred b (a :<: f1) f2 l
-fmlFirst pred b f1 NilFL l = PCs { firsts = reverseRL f1
-                                 , lasts = mapFL_FL ch l}
+fmlFirst pred b f1 NilFL l = PCs { pcsFirsts = reverseRL f1
+                                 , pcsLasts = mapFL_FL ch l}
   where ch (PC tp c) = (PC tp (if pred tp then b else c) )
 
 forceLasts :: Patchy p => [Tag]
@@ -355,8 +367,8 @@ makeEverythingSooner :: forall p C(x y).
   Patchy p => PatchChoices p C(x y) -> PatchChoices p C(x y)
 makeEverythingSooner (PCs f l) =
   case mes NilRL NilRL l
-       of (m :> l) ->
-            PCs (f +>+ m) l
+       of (m :> l') ->
+            PCs (f +>+ m) l'
     where
       mes :: FORALL(m1 m2 m3)
             RL (TaggedPatch p) C(m1 m2) ->
