@@ -15,9 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP , ScopedTypeVariables #-}
---, TypeOperators #-}
 
 #include "gadts.h"
 
@@ -36,17 +34,16 @@ import Data.List ( delete, intersect, (\\) )
 
 import Darcs.Patch ( RepoPatch, getdeps, commute, commuteFLorComplain, commuteRL )
 import Darcs.Patch.Info ( PatchInfo, isTag, humanFriendly )
-import Darcs.Patch.Patchy ( mergeFL )
+import Darcs.Patch.Merge ( mergeFL )
 import Darcs.Patch.Permutations ( partitionFL, partitionRL, removeSubsequenceRL )
-import Darcs.Hopefully( PatchInfoAnd, hopefully, hopefullyM, info )
-import Darcs.Witnesses.Ordered ( (:\/:)(..), (:/\:)(..), (:>)(..), (:>>)(..),
-                                 (=\/=), (=/\=), (+>+), EqCheck(..), mapFL,
+import Darcs.Patch.PatchInfoAnd( PatchInfoAnd, hopefully, hopefullyM, info )
+import Darcs.Witnesses.Eq ( EqCheck(..), (=\/=), (=/\=) )
+import Darcs.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePStart )
+import Darcs.Witnesses.Ordered ( (:\/:)(..), (:/\:)(..), (:>)(..), (:>)(..),
+                                 (+>+), mapFL,
                                  RL(..), FL(..), isShorterThanRL, (+<+),
-                                 reverseFL, reverseRL, mapRL, unsafeCoerceP, unsafeCoercePStart )
+                                 reverseFL, reverseRL, mapRL, )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL )
-#ifdef GADT_WITNESSES
-import Darcs.Patch.Set ( Origin )
-#endif
 import Darcs.ProgressPatches ( progressRL )
 import Darcs.Witnesses.Sealed (Sealed(..), FlippedSeal(..), flipSeal, seal )
 import Printer ( renderString, vcat )
@@ -82,7 +79,7 @@ with_partial_intersection aaa (PatchSet b (Tagged tb _ pb :<: tbs)) j
     = case hopefullyM tb of
         Just _ -> with_partial_intersection aaa (PatchSet (b+<+tb:<:pb) tbs) j
         Nothing -> case splitOnTag (info tb) aaa of
-                   PatchSet NilRL com :>> us -> j com us (unsafeCoerceP b)
+                   PatchSet NilRL com :> us -> j com us (unsafeCoerceP b)
                    _ -> impossible
 
 getPatchesBeyondTag :: RepoPatch p => PatchInfo -> PatchSet p C(start x) -> FlippedSeal (RL (PatchInfoAnd p)) C(x)
@@ -91,7 +88,7 @@ getPatchesBeyondTag t patchset@(PatchSet (hp:<:ps) ts) =
     if info hp == t
     then if getTagsRight patchset == [info hp]
          then flipSeal $ NilRL -- special case to avoid looking at redundant patches
-         else case splitOnTag t patchset of _ :>> e -> flipSeal e
+         else case splitOnTag t patchset of _ :> e -> flipSeal e
     else case getPatchesBeyondTag t (PatchSet ps ts) of
          FlippedSeal xxs -> FlippedSeal (hp :<: xxs)
 getPatchesBeyondTag t (PatchSet NilRL NilRL) = bug $ "tag\n" ++
@@ -100,25 +97,25 @@ getPatchesBeyondTag t (PatchSet NilRL NilRL) = bug $ "tag\n" ++
 getPatchesBeyondTag t0 (PatchSet NilRL (Tagged t _ ps :<: ts)) =
                               getPatchesBeyondTag t0 (PatchSet (t:<:ps) ts)
 
-splitOnTag :: RepoPatch p => PatchInfo -> PatchSet p C(start x) -> (PatchSet p C(start) :>> RL (PatchInfoAnd p)) C(x)
+splitOnTag :: RepoPatch p => PatchInfo -> PatchSet p C(start x) -> (PatchSet p :> RL (PatchInfoAnd p)) C(start x)
 splitOnTag t (PatchSet ps (Tagged hp x ps2 :<: ts))
-    | info hp == t = (PatchSet NilRL (Tagged hp x ps2 :<: ts)) :>> ps
+    | info hp == t = (PatchSet NilRL (Tagged hp x ps2 :<: ts)) :> ps
 splitOnTag t patchset@(PatchSet (hp:<:ps) ts)
     | info hp == t = if getTagsRight patchset == [info hp]
-                     then PatchSet NilRL (Tagged hp Nothing ps :<: ts) :>> NilRL
+                     then PatchSet NilRL (Tagged hp Nothing ps :<: ts) :> NilRL
                      else case partitionRL ((`notElem` (t:ds)) . info) (hp:<:ps) of
                           (x:<:a) :> b ->
                               if getTagsRight (PatchSet (x:<:a) ts) == [t]
-                              then PatchSet NilRL (Tagged x Nothing a :<: ts) :>> b
+                              then PatchSet NilRL (Tagged x Nothing a :<: ts) :> b
                               else case splitOnTag t $ eatOne $ PatchSet (x:<:a) ts of
-                                   xx :>> yy -> xx :>> (b +<+ yy)
-                          -- NilRL :> _ -> impossible
+                                   xx :> yy -> xx :> (b +<+ yy)
+                          _ -> impossible
     where ds = getdeps (hopefully hp)
           eatOne :: PatchSet p C(start x) -> PatchSet p C(start x)
           eatOne (PatchSet ps1 (Tagged x _ ps2 :<: ts')) = PatchSet (ps1+<+x:<:ps2) ts'
           eatOne _ = bug "a stubborn case in splitOnTag (theoretically possible)"
 splitOnTag t (PatchSet (p:<:ps) ts) = case splitOnTag t (PatchSet ps ts) of
-                                        ns :>> x -> ns :>> (p:<:x)
+                                        ns :> x -> ns :> (p:<:x)
 splitOnTag t0 (PatchSet NilRL (Tagged t _ ps :<: ts)) = splitOnTag t0 (PatchSet (t:<:ps) ts)
 splitOnTag t0 (PatchSet NilRL NilRL) = bug $ "tag\n" ++
                                                 renderString (humanFriendly t0) ++
@@ -128,8 +125,16 @@ splitOnTag t0 (PatchSet NilRL NilRL) = bug $ "tag\n" ++
 -- patches in @ps@ which are contained in @t@.
 getPatchesInTag :: RepoPatch p => PatchInfo -> PatchSet p C(start x) -> SealedPatchSet p C(start)
 getPatchesInTag t ns = case splitOnTag t ns of
-                            ns' :>> _ -> seal ns'
+                            ns' :> _ -> seal ns'
 
+-- | @getTagsRight ps@ returns the 'PatchInfo' for all the patches in
+--   @ps@ that are not depended on by anything else *through explicit
+--   dependencies*. Tags are a likely candidate, although we may also
+--   find some non-tag patches in this list.
+--
+--   Keep in mind that in a typical repository with a lot of tags, only a small
+--   fraction of tags would be returned as they would be at least indirectly
+--   depended on by the topmost ones.
 getTagsRight :: PatchSet p C(start x) -> [PatchInfo]
 getTagsRight (PatchSet ps NilRL) = getTagsR (mapRL infoAndDeps ps)
 getTagsRight (PatchSet ps (Tagged t _ _ :<: _)) = getTagsR (mapRL infoAndDeps (ps+<+t:<:NilRL))
@@ -139,6 +144,19 @@ getTagsR [] = []
 getTagsR ((i0,Nothing):pps0) = i0 : getTagsR pps0
 getTagsR ((i0,Just ds0):pps0) = i0 : getTagsR (drop_tags_r ds0 pps0)
     where
+    -- example traversal given dependencies:
+    --
+    -- X1 -> X2
+    -- X1 -> X4
+    -- X2 -> X3
+    --
+    -- [X2,X4]: | o1  o2  X2  o3  o4  X3  o5 X5
+    --            o1 |o2  X2  o3  o4  X3  o5 X5
+    -- [X4,X3]:   o1  o2   |  o3  o4  X3  o5 X5
+    --            ...
+    --    [X4]:   o1  o2      o3  o4   |  o5 X5
+    --            ...
+    --      []:   o1  o2      o3  o4      o5
     drop_tags_r :: [PatchInfo]
                 -> [(PatchInfo, Maybe [PatchInfo])] -> [(PatchInfo, Maybe [PatchInfo])]
     drop_tags_r [] pps = pps
@@ -154,33 +172,53 @@ infoAndDeps p
         | isTag (info p) = (info p, getdeps `fmap` hopefullyM p)
         | otherwise = (info p, Nothing)
 
+-- | @deepOptimizePatchset@ runs through all patches whether they are in the
+--   surface inventory or the deep one.
 deepOptimizePatchset :: PatchSet p C(start x) -> PatchSet p C(start x)
 deepOptimizePatchset ns = optimizePatchset (PatchSet (newset2RL ns) NilRL)
 
+-- | @optimizePatchset@ is similar to @slightlyOptimizePatchset@ in
+--   that it only works on the surface inventory (see below), but it
+--   works a bit harder and can optimize several tags.
+--
+--   * surface inventory: the sequence of patches that corresponds
+--     to the @\_darcs/hashed\_inventory@ or @\_darcs/inventory@ files
+--
+--   * deep inventory: the sequence of sequences of patches that
+--     corresponds to the @\_darcs/inventories@ directory
 optimizePatchset :: PatchSet p C(start x) -> PatchSet p C(start x)
 optimizePatchset (PatchSet NilRL ts) = PatchSet NilRL ts
 optimizePatchset (PatchSet (p:<:ps) ts)
     | isTag (info p) && getTagsRight (PatchSet (p:<:ps) ts) == [info p]
+      -- all subsequent patches depend on p
       = case optimizePatchset (PatchSet ps ts) of
         PatchSet ps' ts' -> PatchSet NilRL (Tagged p Nothing ps' :<: ts')
     | otherwise = case optimizePatchset (PatchSet ps ts) of
                   PatchSet ps' ts' -> PatchSet (p:<:ps') ts'
 
+-- | @slightlyOptimizePatchset@ only works on the surface inventory
+--   (see 'optimizePatchset') and only optimises at most one tag in
+--   there, going for the most recent tag which has no non-depended
+--   patch after it. Older tags won't be 'clean', which means the
+--   PatchSet will not be in 'unclean :< clean' state.
 slightlyOptimizePatchset :: PatchSet p C(start x) -> PatchSet p C(start x)
 slightlyOptimizePatchset (PatchSet ps0 ts0) = sops $ PatchSet (progressRL "Optimizing inventory" ps0) ts0
     where sops :: PatchSet p C(start y) -> PatchSet p C(start y)
           sops (PatchSet NilRL ts) = PatchSet NilRL ts
           sops (PatchSet (hp:<:ps) ts)
               | isTag (info hp) = if getTagsRight (PatchSet (hp:<:ps) ts) == [info hp]
-                                   then PatchSet NilRL (Tagged hp Nothing ps :<: ts)
-                                   else case sops $ PatchSet (progressRL "Optimizing inventory" ps) ts of
+                                   then -- exactly one tag and it depends on everything not already archived
+                                        PatchSet NilRL (Tagged hp Nothing ps :<: ts)
+                                   else -- other tags or other top-level patches too (so move past hp)
+                                        case sops $ PatchSet (progressRL "Optimizing inventory" ps) ts of
                                         PatchSet ps' ts' -> PatchSet (hp:<:ps') ts'
-              | otherwise = case sops $ PatchSet ps ts of
+              | otherwise = -- move past hp
+                            case sops $ PatchSet ps ts of
                             PatchSet ps' ts' -> PatchSet (hp:<:ps') ts'
 
 commuteToEnd :: forall p C(start x y). RepoPatch p => RL (PatchInfoAnd p) C(x y)
-               -> PatchSet p C(start y) -> (PatchSet p C(start) :>> RL (PatchInfoAnd p)) C(x)
-commuteToEnd NilRL (PatchSet ps ts) = PatchSet NilRL ts :>> ps
+               -> PatchSet p C(start y) -> (PatchSet p :> RL (PatchInfoAnd p)) C(start x)
+commuteToEnd NilRL (PatchSet ps ts) = PatchSet NilRL ts :> ps
 commuteToEnd (p:<:ps) (PatchSet xs ts)
     | info p `elem` mapRL info xs = case fastRemoveRL p xs of
                                     Just xs' -> commuteToEnd ps (PatchSet xs' ts)
@@ -203,21 +241,21 @@ removeFromPatchSet bad0 = rfns (reverseFL bad0)
                         rfns bad (PatchSet (ps+<+t:<:tps) ts)
 
 findCommonWithThem :: RepoPatch p => PatchSet p C(start x) -> PatchSet p C(start y)
-                   -> (PatchSet p C(start) :>> FL (PatchInfoAnd p)) C(x)
+                   -> (PatchSet p :> FL (PatchInfoAnd p)) C(start x)
 findCommonWithThem us them =
     with_partial_intersection us them $
     \common us' them' ->
         case partitionFL ((`elem` mapRL info them') . info) $ reverseRL us' of
           _ :> bad@(_:>:_) :> _ -> bug $ "Failed to commute common patches:\n" ++
                                    (renderString $ vcat $ mapRL (humanFriendly . info) $ reverseFL bad)
-          common2 :> _ :> only_ours -> PatchSet (reverseFL common2) common :>> unsafeCoerceP only_ours
+          common2 :> _nilfl :> only_ours -> PatchSet (reverseFL common2) common :> unsafeCoerceP only_ours
 
 findUncommon :: RepoPatch p => PatchSet p C(start x) -> PatchSet p C(start y)
                          -> (FL (PatchInfoAnd p) :\/: FL (PatchInfoAnd p)) C(x y)
 findUncommon us them =
   case findCommonWithThem us them of
-    common :>> us' -> case findCommonWithThem them us of
-      _ :>> them' -> unsafeCoercePStart us' :\/: them'
+    _common :> us' -> case findCommonWithThem them us of
+      _ :> them' -> unsafeCoercePStart us' :\/: them'
 
 countUsThem :: RepoPatch p => PatchSet p C(start x) -> PatchSet p C(start y) -> (Int, Int)
 countUsThem us them =
