@@ -16,9 +16,8 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts -fno-warn-orphans #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE CPP #-}
--- , TypeOperators, PatternGuards #-}
 
 #include "gadts.h"
 
@@ -30,17 +29,24 @@ module Darcs.Patch.Permutations ( removeFL, removeRL, removeCommon,
                                   headPermutationsFL,
                                   removeSubsequenceFL, removeSubsequenceRL,
                                   partitionConflictingFL,
-                                  CommuteFn, selfCommuter, commuterIdRL,
+                                  CommuteFn, selfCommuter,
+                                  commuterIdFL, commuterFLId,
+                                  commuterIdRL
                                 ) where
 
-import Data.Maybe ( catMaybes )
-import Darcs.Patch.Patchy ( Commute, commute, commuteFLorComplain, commuteRL, Invert(..), invertFL, invertRL )
-import Darcs.Witnesses.Ordered
+import Data.Maybe ( mapMaybe )
+import Darcs.Patch.Commute ( Commute, commute, commuteFLorComplain, commuteRL )
+import Darcs.Patch.Invert ( Invert(..), invertFL, invertRL )
+import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), (+<+)
+                               , reverseFL, (+>+), (:\/:)(..), lengthFL
+                               , lengthRL, reverseRL )
 #include "impossible.h"
 
--- |split an 'FL' into "left" and "right" lists according to a predicate, using commutation as necessary.
+-- |split an 'FL' into "left" and "right" lists according to a predicate @p@, using commutation as necessary.
 -- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
--- the predicate, it goes in the "right" list.
+-- the predicate, it goes in the "middle" list; to sum up, we have: @all p left@ and @all (not.p) right@, while
+-- midddle is mixed.
 partitionFL :: Commute p
             => (FORALL(u v) p C(u v) -> Bool)       -- ^predicate; if true we would like the patch in the "left" list
             -> FL p C(x y)                          -- ^input 'FL'
@@ -192,7 +198,7 @@ simpleHeadPermutationsFL ps = map (\ (x:>xs) -> x:>:xs) $ headPermutationsFL ps
 headPermutationsFL :: Commute p => FL p C(x y) -> [(p :> FL p) C(x y)]
 headPermutationsFL NilFL = []
 headPermutationsFL (p:>:ps) =
-    (p:>ps) : catMaybes (map (swapfirstFL.(p:>)) $ headPermutationsFL ps)
+    (p:>ps) : mapMaybe (swapfirstFL.(p:>)) (headPermutationsFL ps)
         where swapfirstFL (p1:>p2:>xs) = do p2':>p1' <- commute (p1:>p2)
                                             Just $ p2':>p1':>:xs
 
@@ -202,7 +208,7 @@ headPermutationsFL (p:>:ps) =
 headPermutationsRL :: Commute p => RL p C(x y) -> [RL p C(x y)]
 headPermutationsRL NilRL = []
 headPermutationsRL (p:<:ps) =
-    (p:<:ps) : catMaybes (map (swapfirstRL.(p:<:)) $ headPermutationsRL ps)
+    (p:<:ps) : mapMaybe (swapfirstRL.(p:<:)) (headPermutationsRL ps)
         where swapfirstRL (p1:<:p2:<:xs) = do p1':>p2' <- commute (p2:>p1)
                                               Just $ p2':<:p1':<:xs
               swapfirstRL _ = Nothing
@@ -218,10 +224,6 @@ instance (MyEq p, Commute p) => MyEq (FL p) where
 
 instance (Invert p, Commute p) => Invert (FL p) where
     invert = reverseRL . invertFL
-    identity = NilFL
-    sloppyIdentity NilFL = IsEq
-    sloppyIdentity (x:>:xs) | IsEq <- sloppyIdentity x = sloppyIdentity xs
-    sloppyIdentity _ = NotEq
 
 instance (MyEq p, Commute p) => MyEq (RL p) where
     unsafeCompare = bug "Buggy use of unsafeCompare on RL"
@@ -235,10 +237,6 @@ instance (MyEq p, Commute p) => MyEq (RL p) where
 
 instance (Commute p, Invert p) => Invert (RL p) where
     invert = reverseFL . invertRL
-    identity = NilRL
-    sloppyIdentity NilRL = IsEq
-    sloppyIdentity (x:<:xs) | IsEq <- sloppyIdentity x = sloppyIdentity xs
-    sloppyIdentity _ = NotEq
 
 -- |CommuteFn is the basis of a general framework for building up commutation
 -- operations between different patch types in a generic manner. Unfortunately
@@ -261,6 +259,20 @@ commuterIdRL commuter (x :> (y :<: ys))
        y' :> x'' <- commuter (x' :> y)
        return ((y' :<: ys') :> x'')
 
+commuterIdFL :: CommuteFn p1 p2 -> CommuteFn p1 (FL p2)
+commuterIdFL _ (x :> NilFL) = return (NilFL :> x)
+commuterIdFL commuter (x :> (y :>: ys))
+  = do y' :> x' <- commuter (x :> y)
+       ys' :> x'' <- commuterIdFL commuter (x' :> ys)
+       return ((y' :>: ys') :> x'')
+
+commuterFLId :: CommuteFn p1 p2 -> CommuteFn (FL p1) p2
+commuterFLId _ (NilFL :> y) = return (y :> NilFL)
+commuterFLId commuter ((x :>: xs) :> y)
+  = do y' :> xs' <- commuterFLId commuter (xs :> y)
+       y'' :> x' <- commuter (x :> y')
+       return (y'' :> (x' :>: xs'))
+
 -- |Partition a list into the patches that commute with the given patch and those that don't (including dependencies)
 partitionConflictingFL :: (Commute p1, Invert p1) => CommuteFn p1 p2 -> FL p1 C(x y) -> p2 C(x z) -> (FL p1 :> FL p1) C(x y)
 partitionConflictingFL _ NilFL _ = (NilFL :> NilFL)
