@@ -15,7 +15,6 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp #-}
 {-# LANGUAGE CPP #-}
 
 #include "gadts.h"
@@ -24,9 +23,11 @@ module Darcs.PrintPatch ( printPatch, contextualPrintPatch,
                           printPatchPager, printFriendly ) where
 
 import Darcs.Patch ( Patchy, showContextPatch, showPatch )
+import Darcs.Patch.Apply ( ApplyState )
 import Storage.Hashed.Tree( Tree )
 import Storage.Hashed.Monad( virtualTreeIO )
 import Darcs.Arguments ( DarcsFlag, showFriendly )
+import Darcs.Flags ( isUnified )
 import Printer ( putDocLnWith )
 import Darcs.ColorPrinter ( fancyPrinters )
 import Darcs.External ( viewDocWith )
@@ -34,8 +35,10 @@ import Darcs.External ( viewDocWith )
 -- | @'printFriendly' opts patch@ prints @patch@ in accordance with the
 -- flags in opts, ie, whether @--verbose@ or @--summary@ were passed at
 -- the command-line.
-printFriendly :: Patchy p => [DarcsFlag] -> p C(x y) -> IO ()
-printFriendly opts p = putDocLnWith fancyPrinters $ showFriendly opts p
+printFriendly :: (Patchy p, ApplyState p ~ Tree)  => (Maybe (Tree IO)) -> [DarcsFlag] -> p C(x y) -> IO ()
+printFriendly (Just pristine) opts p
+ | isUnified opts = virtualTreeIO (showContextPatch p) pristine >>= putDocLnWith fancyPrinters . fst
+printFriendly _ opts p = putDocLnWith fancyPrinters $ showFriendly opts p
 
 -- | 'printPatch' prints a patch on standard output.
 printPatch :: Patchy p => p C(x y) -> IO ()
@@ -47,5 +50,5 @@ printPatchPager p = viewDocWith fancyPrinters $ showPatch p
 
 -- | 'contextualPrintPatch' prints a patch, together with its context,
 -- on standard output.
-contextualPrintPatch :: Patchy p => Tree IO -> p C(x y) -> IO ()
+contextualPrintPatch :: (Patchy p, ApplyState p ~ Tree) => Tree IO -> p C(x y) -> IO ()
 contextualPrintPatch s p = virtualTreeIO (showContextPatch p) s >>= putDocLnWith fancyPrinters . fst
