@@ -1,4 +1,3 @@
-{-# OPTIONS_GHC -cpp #-}
 {-# LANGUAGE CPP #-}
 
 -- Copyright (C) 2007 Eric Kow
@@ -47,8 +46,10 @@ module Darcs.RepoPath (
   setCurrentDirectory
 ) where
 
+import Control.Applicative ( (<$>) )
 import Data.List ( isPrefixOf, isSuffixOf )
-import Control.Exception ( bracket )
+import Control.Exception ( tryJust, bracket )
+import System.IO.Error ( isDoesNotExistError )
 
 import Darcs.URL ( isAbsolute, isRelative, isSshNopath )
 import qualified Workaround ( getCurrentDirectory )
@@ -57,14 +58,13 @@ import System.Directory ( doesDirectoryExist )
 import qualified System.FilePath.Posix as FilePath ( normalise )
 import qualified System.FilePath as NativeFilePath ( takeFileName, takeDirectory )
 import qualified Darcs.Patch.FileName as PatchFileName ( FileName, fp2fn, fn2fp )
+import System.Posix.Files ( isDirectory, getSymbolicLinkStatus )
 #include "impossible.h"
 
 class FilePathOrURL a where
- {-# INLINE toPath #-}
  toPath :: a -> String
 
 class FilePathOrURL a => FilePathLike a where
- {-# INLINE toFilePath #-}
  toFilePath :: a -> FilePath
 
 -- | Paths which are relative to the local darcs repository and normalized.
@@ -123,10 +123,19 @@ simpleSubPath x | null x = bug "simpleSubPath called with empty path"
                 | isRelative x = Just $ SubPath $ FilePath.normalise $ pathToPosix x
                 | otherwise = Nothing
 
+-- | Ensure directory exists and is not a symbolic link.
+doesDirectoryReallyExist :: FilePath -> IO Bool
+doesDirectoryReallyExist f = do
+    x <- tryJust (\x -> if isDoesNotExistError x then Just () else Nothing) $
+        isDirectory <$> getSymbolicLinkStatus f
+    return $ case x of
+        Left () -> False
+        Right y -> y
+
 -- | Interpret a possibly relative path wrt the current working directory.
 ioAbsolute :: FilePath -> IO AbsolutePath
 ioAbsolute dir =
-    do isdir <- doesDirectoryExist dir
+    do isdir <- doesDirectoryReallyExist dir
        here <- getCurrentDirectory
        if isdir
          then bracket (setCurrentDirectory dir)
