@@ -1,4 +1,3 @@
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP #-}
 
 #include "gadts.h"
@@ -12,12 +11,13 @@ module Darcs.Repository.Cache (
                    writeFileUsingCache,
                    peekInCache,
                    repo2cache,
-                   writable, isthisrepo, hashedFilePath, allHashedDirs, compareByLocality
+                   writable, isthisrepo, hashedFilePath, allHashedDirs, compareByLocality,
+                   reportBadSources
                  ) where
 
-import Control.Monad ( liftM, when, guard, unless, filterM, forM_ )
-import Data.List ( nub )
-import Data.Maybe ( catMaybes, listToMaybe )
+import Control.Monad ( liftM, when, guard, unless, filterM, forM_, mplus )
+import Data.List ( nub, intercalate )
+import Data.Maybe ( catMaybes, listToMaybe, isJust, fromMaybe )
 import System.Directory ( removeFile, doesFileExist, doesDirectoryExist,
                           getDirectoryContents, getPermissions )
 import qualified System.Directory as SD ( writable )
@@ -32,16 +32,22 @@ import qualified Data.ByteString.Char8 as BC (unpack)
 
 import SHA1 ( sha1PS )
 import System.Posix.Files ( createLink )
+import System.FilePath.Posix ( (</>) )
 import System.Directory ( createDirectoryIfMissing )
 
 import Darcs.External ( gzFetchFilePS, fetchFilePS, speculateFileOrUrl, copyFileOrUrl,
                         Cachable( Cachable ) )
-import Darcs.Flags ( Compression( .. ) )
-import Darcs.Global ( darcsdir )
-import Darcs.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS )
+import Darcs.Flags ( Compression( .. ), RemoteDarcs(..) )
+import Darcs.Global ( darcsdir, addBadSource, isBadSource,
+                      addReachableSource, isReachableSource,
+                      getBadSourcesList )
+import Darcs.Lock ( writeAtomicFilePS, gzWriteAtomicFilePS, withTemp )
 import Progress ( progressList, debugMessage, debugFail )
-import Darcs.URL ( isFile, isUrl, isSsh )
+import English ( englishNum, Noun(..), Pronoun(..) )
+import Darcs.URL ( isFile, isHttpUrl, isSshUrl )
 import Darcs.Utils ( withCurrentDirectory, catchall )
+import Darcs.SignalHandler ( catchNonSignal )
+import qualified URL ( ConnectionError(..) )
 
 data HashedDir = HashedPristineDir | HashedPatchesDir | HashedInventoriesDir
 hashedDir :: HashedDir -> String
@@ -54,7 +60,7 @@ allHashedDirs = [HashedPristineDir, HashedPatchesDir, HashedInventoriesDir]
 
 data WritableOrNot = Writable | NotWritable deriving ( Show )
 data CacheType = Repo | Directory deriving ( Eq, Show )
-data CacheLoc = Cache !CacheType !WritableOrNot !String
+data CacheLoc = Cache { cacheType:: !CacheType, cacheWritable:: !WritableOrNot, cacheSource:: !String }
 newtype Cache = Ca [CacheLoc] -- abstract type for hiding cache
 
 instance Eq CacheLoc where
@@ -113,11 +119,11 @@ compareByLocality :: CacheLoc -> CacheLoc -> Ordering
 compareByLocality (Cache _ _  x) (Cache _ _ y)
   | isLocal x &&  isRemote y  = LT
   | isRemote x && isLocal y = GT
-  | isUrl x && isSsh y = LT
-  | isSsh x && isUrl y = GT
+  | isHttpUrl x && isSshUrl y = LT
+  | isSshUrl x && isHttpUrl y = GT
   | otherwise = EQ
     where
-      isRemote r= isUrl r || isSsh r
+      isRemote r= isHttpUrl r || isSshUrl r
       isLocal = isFile
 
 repo2cache :: String -> Cache
@@ -127,7 +133,7 @@ repo2cache r = Ca [Cache Repo NotWritable r]
 cacheHash :: B.ByteString -> String
 cacheHash ps = case show (B.length ps) of
                  x | l > 10 -> sha256sum ps
-                   | otherwise -> take (10-l) (repeat '0') ++ x ++'-':sha256sum ps
+                   | otherwise -> replicate (10-l) '0' ++ x ++'-':sha256sum ps
                                         where l = length x
 
 okayHash :: String -> Bool
@@ -145,7 +151,11 @@ checkHash h s | length h == 40 = sha1PS s == h
               | length h == 75 = B.length s == read (take 10 h) && sha256sum s == drop 11 h
               | otherwise = False
 
-
+-- |@fetchFileUsingCache cache dir hash@ receives a list of caches
+--  @cache@, the directory for which that file belongs @dir@ and the
+--  @hash@ of the file to fetch.  It tries to fetch the file from one
+--  of the sources, trying them in order one by one.  If the file
+--  cannot be fetched from any of the sources, this operation fails.
 fetchFileUsingCache :: Cache -> HashedDir -> String -> IO (String, B.ByteString)
 fetchFileUsingCache = fetchFileUsingCachePrivate Anywhere
 
@@ -195,12 +205,15 @@ speculateFilesUsingCache cache sd hs =
 
 data OrOnlySpeculate = ActuallyCopy | OnlySpeculate deriving ( Eq )
 
+-- | @copyFileUsingCache oos cache subdir f@ copy or hardlinks the
+--   file @f@ to one of the caches in @cache@,usually the global cache.
+--   If a writable cache doesn't exist the function will fail.
 copyFileUsingCache :: OrOnlySpeculate -> Cache -> HashedDir -> String -> IO ()
 copyFileUsingCache oos (Ca cache) subdir f =
     do debugMessage $ "I'm doing copyFileUsingCache on "++(hashedDir subdir)++"/"++f
        Just stickItHere <- cacheLoc cache
        createDirectoryIfMissing False (reverse $ dropWhile (/='/') $ reverse stickItHere)
-       sfuc cache stickItHere
+       filterBadSources cache >>= (flip sfuc) stickItHere
     `catchall` return ()
     where cacheLoc [] = return Nothing
           cacheLoc (c:cs) | not $ writable c = cacheLoc cs
@@ -208,28 +221,92 @@ copyFileUsingCache oos (Ca cache) subdir f =
               do ex <- doesFileExist $ fn c
                  if ex then fail "Bug in darcs: This exception should be caught in speculateFileUsingCache"
                        else do othercache <- cacheLoc cs
-                               case othercache of Just x -> return $ Just x
-                                                  Nothing -> return $ Just (fn c)
+                               return $ othercache `mplus` Just (fn c)
           sfuc [] _ = return ()
-          sfuc (c:cs) out | not $ writable c =
+          sfuc (c:cs) out
+            | not (writable c) =
               if oos == OnlySpeculate
-                 then speculateFileOrUrl (fn c) out
-                 else copyFileOrUrl [] (fn c) out Cachable
-                          | otherwise = sfuc cs out
+               then speculateFileOrUrl (fn c) out `catchNonSignal` (\e -> checkCacheReachability (show e) c)
+               else copyFileOrUrl DefaultRemoteDarcs (fn c) out Cachable `catchNonSignal` (\e -> checkCacheReachability (show e) c)
+            | otherwise = sfuc cs out
           fn c = hashedFilePath c subdir f
 
 copyFilesUsingCache :: OrOnlySpeculate -> Cache -> HashedDir -> [String] -> IO ()
 copyFilesUsingCache oos cache subdir hs =
-    do forM_ hs $ copyFileUsingCache oos cache subdir
+    forM_ hs $ copyFileUsingCache oos cache subdir
 
 
 data FromWhere = LocalOnly | Anywhere deriving ( Eq )
 
+-- | Checks if a given cache entry is reachable or not.
+-- It receives an error caught during execution and the cache entry.
+-- If the caches is not reachable it is blacklisted and not longer tried for
+-- the rest of the session. If it is reachable it is whitelisted and future errors with such
+-- cache get ignore.
+-- To determine reachability:
+--  * For a local cache, if the given source doesn't exist anymore, it is blacklisted.
+--  * For remote sources if the error is timeout, it is blacklisted, if not,
+--    it checks if _darcs/hashed_inventory  exist, if it does, the entry is whitelisted, if
+--    it doesn't, it is blacklisted.
+checkCacheReachability :: String -> CacheLoc -> IO ()
+checkCacheReachability e cache
+ | isFile source = do
+     reachable <- isReachableSource
+     unless (reachable source) $ do
+       exist <- doesDirectoryExist source
+       if exist
+        then
+         addReachableSource source
+        else
+         addBadSource source
+ | isHttpUrl source = do
+     reachable <- isReachableSource
+     unless (reachable source) $ do
+            let string = case dropWhile (/='(') e of
+                          (_:xs) -> fst (break (==')') xs)
+                          _      -> e
+            let cerror = case reads string ::[(URL.ConnectionError,String)] of
+                           [(ce,_)] -> Just ce
+                           _        -> Nothing
+            if isJust cerror
+             then addBadSource source
+             else checkFileReachability
+
+ | isSshUrl source = do
+   reachable <- isReachableSource
+   unless (reachable source) checkFileReachability
+
+ | otherwise = fail $ "unknown transport protocol for: " ++ source
+ where source = cacheSource cache
+       checkFileReachability = do
+         reachable <- checkHashedInventoryReachability cache
+         if reachable
+          then
+           addReachableSource source
+          else
+           addBadSource source
+
+-- | Returns a list of reachables cache entries,
+--   taking out the blacklisted entries
+filterBadSources :: [CacheLoc] -> IO [CacheLoc]
+filterBadSources cache = do
+   badSource <- isBadSource
+   return $ filter (not . badSource . cacheSource) cache
+
+-- | Checks if the  _darcs/hashed_inventory exist and is reachable
+checkHashedInventoryReachability :: CacheLoc -> IO Bool
+checkHashedInventoryReachability cache =
+     withTemp $ \tempout -> do
+       let f = cacheSource cache </> darcsdir </> "hashed_inventory"
+       copyFileOrUrl DefaultRemoteDarcs f tempout Cachable
+       return True
+     `catchNonSignal` (\_ -> return False)
+
 fetchFileUsingCachePrivate :: FromWhere -> Cache -> HashedDir -> String -> IO (String, B.ByteString)
 fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f =
     do when (fromWhere == Anywhere) $ copyFileUsingCache ActuallyCopy (Ca cache) subdir f
-       ffuc cache
-    `catchall` debugFail ("Couldn't fetch `"++f++"'\nin subdir "++(hashedDir subdir)++
+       filterBadSources cache >>= ffuc
+    `catchall` debugFail ("Couldn't fetch `"++f++"'\nin subdir "++ hashedDir subdir ++
                           " from sources:\n\n"++show (Ca cache))
     where ffuc (c:cs)
            | not (writable c) && (Anywhere == fromWhere || isFile (fn c)) =
@@ -244,7 +321,10 @@ fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f =
                                     fail $ "Hash failure in " ++ fn c
                             return (fn c, x')
                     else return (fn c, x) -- FIXME: create links in caches
-              `catchall` ffuc cs
+              `catchNonSignal` (\e -> do
+                                      checkCacheReachability (show e) c
+                                      filterBadSources cs >>= ffuc)
+
 
            | writable c =
               do x1 <- gzFetchFilePS (fn c) Cachable
@@ -258,13 +338,15 @@ fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f =
                       else return x1
                  mapM_ (tryLinking (fn c)) cs
                  return (fn c, x)
-              `catchall` do (fname,x) <- ffuc cs
-                            do createCache c subdir
-                               createLink fname (fn c)
-                               return (fn c, x)
-                             `catchall`
-                             do gzWriteFilePS (fn c) x `catchall` return ()
-                                return (fname,x)
+              `catchNonSignal` (\ e ->
+                                 do createCache c subdir `catchall` return () -- don't worry
+                                    checkCacheReachability (show e) c
+                                    (fname,x) <- filterBadSources cs >>= ffuc
+                                    do createLink fname (fn c)
+                                       return (fn c, x)
+                                     `catchall`
+                                      do gzWriteFilePS (fn c) x `catchall` return ()
+                                         return (fname,x))
            | otherwise = ffuc cs
 
           ffuc [] = debugFail $ "No sources from which to fetch file `"++f++"'\n"++ show (Ca cache)
@@ -316,12 +398,23 @@ cleanCachesWithHint' (Ca cs) subdir hint = mapM_ cleanCache cs
     where cleanCache (Cache Directory Writable d) =
              (withCurrentDirectory (d++"/"++(hashedDir subdir)) $
               do fs' <- getDirectoryContents "."
-                 let fs = case hint of
-                            Just h -> h
-                            Nothing -> fs'
+                 let fs = fromMaybe fs' hint
                  mapM_ clean $ progressList ("Cleaning cache "++d++"/"++(hashedDir subdir)) $
                        filter okayHash fs) `catchall` return ()
           cleanCache _ = return ()
           clean f = do lc <- linkCount `liftM` getSymbolicLinkStatus f
                        when (lc < 2) $ removeFile f
                     `catchall` return ()
+
+-- | Prints an error message with a list of bad caches.
+reportBadSources :: IO ()
+reportBadSources = do
+  sources <- getBadSourcesList
+  let size = length sources
+  unless (null sources) $
+   hPutStrLn stderr $
+    concat [ "\nHINT: I could not reach the following ", englishNum size  (Noun "repository") ":"
+           , "\n", (intercalate "\n" (map ("        " ++) sources))
+           , "\n      If you're not using ", englishNum size It ", you should probably delete"
+           , "\n      the corresponding ", englishNum size (Noun "entry") " from _darcs/prefs/sources."
+           ]
