@@ -14,9 +14,7 @@
 -- along with this program; if not, write to the Free Software Foundation,
 -- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
 {-# LANGUAGE CPP #-}
--- , KindSignatures #-}
 
 #include "gadts.h"
 
@@ -31,12 +29,13 @@ import Darcs.Repository.Format ( RepoFormat )
 import Darcs.Patch ( RepoPatch )
 
 data Pristine
-  = NoPristine !String
-  | PlainPristine !String
+  = NoPristine
+  | PlainPristine
   | HashedPristine
-    deriving ( Show )
+    deriving ( Show, Eq )
 
-data Repository (p :: * C(-> * -> *)) C(recordedstate unrecordedstate tentativestate) = Repo !String ![DarcsFlag] !RepoFormat !(RepoType p) deriving ( Show )
+data Repository (p :: * C(-> * -> *)) C(recordedstate unrecordedstate tentativestate) =
+  Repo !String ![DarcsFlag] !RepoFormat !(RepoType p) deriving ( Show )
 
 data RepoType (p :: * C(-> * -> *)) = DarcsRepository !Pristine Cache deriving ( Show )
 
@@ -51,4 +50,4 @@ extractOptions (Repo _ opts _ _) = opts
 modifyCache :: FORALL(p r u t) (RepoPatch p)  => Repository p C(r u t) -> (Cache -> Cache) -> Repository p C(r u t)
 modifyCache (Repo dir opts rf (DarcsRepository pristine cache)) f = Repo dir opts rf dr
   where dr            = DarcsRepository pristine . cmap ( sortBy compareByLocality . nub ) $ f cache
-        cmap f (Ca c) = Ca (f c)
+        cmap g (Ca c) = Ca (g c)
