@@ -34,7 +34,11 @@ module Darcs.Repository.Prefs ( addToPreflist, getPreflist, setPreflist,
                    globalPrefsDirDoc,
                  ) where
 
+
+import Prelude hiding ( catch )
+
 import System.IO.Error ( isDoesNotExistError )
+import Control.Exception ( catch )
 import Control.Monad ( unless, when )
 import Text.Regex ( Regex, mkRegex, matchRegex, )
 import Data.Char ( toUpper )
