@@ -15,9 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
-{-# LANGUAGE CPP #-}
--- , DeriveDataTypeable #-}
+{-# LANGUAGE CPP, DeriveDataTypeable #-}
 
 module Darcs.SignalHandler ( withSignalsHandled, withSignalsBlocked,
                              catchInterrupt, catchNonSignal,
@@ -29,7 +27,7 @@ import System.IO.Error ( isUserError, ioeGetErrorString, ioeGetFileName )
 import System.Exit ( exitWith, ExitCode ( ExitFailure ) )
 import Control.Concurrent ( ThreadId, myThreadId )
 import Control.Exception.Extensible
-            ( catch, throw, throwTo, block,
+            ( catch, throw, throwTo, block, unblock,
               Exception(..), SomeException(..), IOException )
 import System.Posix.Files ( getFdStatus, isNamedPipe )
 import System.Posix.IO ( stdOutput )
@@ -88,12 +86,14 @@ withSignalsHandled job = do
 #endif
 
 resethandler :: Signal -> IO ()
-resethandler s = do installHandler s Default Nothing
+-- Warning:  A do-notation statement discarded a result of type Handler.
+resethandler s = do _ <- installHandler s Default Nothing
                     return ()
 
 ih :: ThreadId -> Signal -> IO ()
 ih thid s =
-  do installHandler s (Catch $ throwTo thid $ SignalException s) Nothing
+-- Warning:  A do-notation statement discarded a result of type Handler.
+  do _ <- installHandler s (Catch $ throwTo thid $ SignalException s) Nothing
      return ()
 
 catchSignal :: IO a -> (Signal -> IO a) -> IO a
@@ -127,14 +127,16 @@ catchUserErrors comp handler = catch comp handler'
          | ioeGetFileName ioe == Just "<stdout>" = handler ("STDOUT" ++ ioeGetErrorString ioe)
          | otherwise                             = throw ioe
 
-withSignalsBlocked :: IO () -> IO ()
-withSignalsBlocked job = (block job) `catchSignal` couldnt_do
-    where couldnt_do s | s == sigINT = oops "interrupt"
-                       | s ==  sigHUP = oops "HUP"
-                       | s ==  sigABRT = oops "ABRT"
-                       | s ==  sigALRM = oops "ALRM"
-                       | s ==  sigTERM = oops "TERM"
-                       | s ==  sigPIPE = return ()
-                       | otherwise = oops "unknown signal"
-          oops s = hPutStrLn stderr $ "Couldn't handle " ++ s ++
-                   " since darcs was in a sensitive job."
+withSignalsBlocked :: IO a -> IO a
+withSignalsBlocked job = block (job >>= \r ->
+                           unblock(return r) `catchSignal` couldnt_do r)
+    where couldnt_do r s | s == sigINT = oops "interrupt" r
+                         | s ==  sigHUP = oops "HUP" r
+                         | s ==  sigABRT = oops "ABRT" r
+                         | s ==  sigALRM = oops "ALRM" r
+                         | s ==  sigTERM = oops "TERM" r
+                         | s ==  sigPIPE = return r
+                         | otherwise = oops "unknown signal" r
+          oops s r = do hPutStrLn stderr $ "Couldn't handle " ++ s ++
+                          " since darcs was in a sensitive job."
+                        return r
