@@ -32,6 +32,8 @@ import Data.Map ( Map )
 import qualified Data.Map as M ( mapKeys, delete, insert, empty, lookup, null )
 import System.FilePath ( joinPath, splitDirectories )
 
+#include "impossible.h"
+
 -- | File contents are represented by a map from line numbers to line contents.
 --   If for a certain line number, the line contents are Nothing, that means
 --   that we are sure that that line exists, but we don't know its contents.
@@ -58,6 +60,12 @@ instance  Show Prop  where
     show (NotEx f) = "NotEx"++f
     show (FileLines f l)  = "FileLines "++f++": "++show l
 
+-- TODO the way that the standard way to use PatchCheck is
+-- by returning PatchCheck Bool but then often ignoring the
+-- result and instead checking again for state consistency
+-- is weird. It should be possible to replace it by a more normal
+-- error handling mechanism.
+
 -- | PatchCheck is a state monad with a simulated repository state
 type PatchCheck = State KnownState
 
@@ -82,7 +90,7 @@ doCheck p = evalState p (P [] [])
 doVerboseCheck :: PatchCheck a -> a
 doVerboseCheck p =
     case runState p (P [] []) of
-    (b, pc) -> unsafePerformIO $ do putStrLn $ show pc
+    (b, pc) -> unsafePerformIO $ do print pc
                                     return b
 
 -- | Returns true if the current repository state is not inconsistent
@@ -97,7 +105,7 @@ modifyFile :: String
             -> (Maybe FileContents -> Maybe FileContents)
             -> PatchCheck Bool
 modifyFile f change = do
-    fileExists f
+    _ <- fileExists f
     c <- fileContents f
     case change c of
       Nothing -> assertNot $ FileEx f -- shorthand for "FAIL"
@@ -166,7 +174,7 @@ fileEmpty f = do
   let empty = case c of
                Just c' -> fcMaxline c' == 0 && M.null (fcLines c')
                Nothing -> True
-  if empty
+  _ <- if empty
      then do setContents f emptyFilecontents
              isValid
      -- Crude way to make it inconsistent and return false:
@@ -183,7 +191,7 @@ movedirfilename d d' f
 --   is consistent, False if it is not.
 doSwap :: String -> String -> PatchCheck Bool
 doSwap f f' = handleInconsistent False $ do
-    modify (\(P ks nots) -> P (map sw ks) (map sw nots))
+    modify map_sw
     return True
   where sw (FileEx a) | f  `is_soe` a = FileEx $ movedirfilename f f' a
                       | f' `is_soe` a = FileEx $ movedirfilename f' f a
@@ -196,6 +204,8 @@ doSwap f f' = handleInconsistent False $ do
         sw p = p
         is_soe d1 d2 = -- is_superdir_or_equal
             d1 == d2 || (d1 ++ "/") `isPrefixOf` d2
+        map_sw (P ks nots) = P (map sw ks) (map sw nots)
+        map_sw _ = impossible
 
 -- | Assert a property about the repository. If the property is already present
 -- in the repo state, nothing changes, and the function returns True. If it is
@@ -235,70 +245,74 @@ assertNot p = handleInconsistent False $ do
 -- Returns False if the repo is inconsistent, True otherwise.
 changeToTrue :: Prop -> PatchCheck Bool
 changeToTrue p = handleInconsistent False $ do
-    modify (\(P ks nots) -> P (p:ks) (filter (p /=) nots))
+    modify filter_nots
     return True
+      where filter_nots (P ks nots) = P (p:ks) (filter (p /=) nots)
+            filter_nots _ = impossible
 
 -- | Remove a property from the list of properties that hold for this repo (if
 -- it's in there), and add it to the list of properties that do not hold.
 -- Returns False if the repo is inconsistent, True otherwise.
 changeToFalse :: Prop -> PatchCheck Bool
 changeToFalse p = handleInconsistent False $ do
-    modify (\(P ks nots) -> P (filter (p /=) ks) (p:nots))
+    modify filter_ks
     return True
+    where filter_ks (P ks nots) = P (filter (p /=) ks) (p:nots)
+          filter_ks _ = impossible
 
 assertFileExists :: String -> PatchCheck Bool
-assertFileExists f =   do assertNot $ NotEx f
-                          assertNot $ DirEx f
+assertFileExists f =   do _ <- assertNot $ NotEx f
+                          _ <- assertNot $ DirEx f
                           assert $ FileEx f
 assertDirExists :: String -> PatchCheck Bool
-assertDirExists d =   do assertNot $ NotEx d
-                         assertNot $ FileEx d
+assertDirExists d =   do _ <- assertNot $ NotEx d
+                         _ <- assertNot $ FileEx d
                          assert $ DirEx d
 assertExists :: String -> PatchCheck Bool
 assertExists f = assertNot $ NotEx f
 
 assertNoSuch :: String -> PatchCheck Bool
-assertNoSuch f =   do assertNot $ FileEx f
-                      assertNot $ DirEx f
+assertNoSuch f =   do _ <- assertNot $ FileEx f
+                      _ <- assertNot $ DirEx f
                       assert $ NotEx f
 
 createFile :: String -> PatchCheck Bool
 createFile fn = do
-  superdirsExist fn
-  assertNoSuch fn
-  changeToTrue (FileEx fn)
+  _ <- superdirsExist fn
+  _ <- assertNoSuch fn
+  _ <- changeToTrue (FileEx fn)
   changeToFalse (NotEx fn)
 
 createDir :: String -> PatchCheck Bool
 createDir fn = do
-  substuffDontExist fn
-  superdirsExist fn
-  assertNoSuch fn
-  changeToTrue (DirEx fn)
+  _ <- substuffDontExist fn
+  _ <- superdirsExist fn
+  _ <- assertNoSuch fn
+  _ <- changeToTrue (DirEx fn)
   changeToFalse (NotEx fn)
 
 removeFile :: String -> PatchCheck Bool
 removeFile fn = do
-  superdirsExist fn
-  assertFileExists fn
-  fileEmpty fn
-  changeToFalse (FileEx fn)
+  _ <- superdirsExist fn
+  _ <- assertFileExists fn
+  _ <- fileEmpty fn
+  _ <- changeToFalse (FileEx fn)
   changeToTrue (NotEx fn)
 
 removeDir :: String -> PatchCheck Bool
 removeDir fn = do
-  substuffDontExist fn
-  superdirsExist fn
-  assertDirExists fn
-  changeToFalse (DirEx fn)
+  _ <- substuffDontExist fn
+  _ <- superdirsExist fn
+  _ <- assertDirExists fn
+  _ <- changeToFalse (DirEx fn)
   changeToTrue (NotEx fn)
 
 checkMove :: String -> String -> PatchCheck Bool
 checkMove f f' = do
-  superdirsExist f
-  superdirsExist f'
-  assertExists f
-  assertNoSuch f'
+  _ <- superdirsExist f
+  _ <- superdirsExist f'
+  _ <- assertExists f
+  _ <- assertNoSuch f'
   doSwap f f'
 
 substuffDontExist :: String -> PatchCheck Bool
@@ -324,10 +338,10 @@ superdirsExist fn = and `fmap` mapM assertDirExists superdirs
 
 fileExists :: String -> PatchCheck Bool
 fileExists fn = do
-  superdirsExist fn
+  _ <- superdirsExist fn
   assertFileExists fn
 
 dirExists :: String -> PatchCheck Bool
 dirExists fn = do
-  superdirsExist fn
+  _ <- superdirsExist fn
   assertDirExists fn
