@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-dodgy-imports #-} -- needed for GHC 7.0/7.2
 {-# LANGUAGE CPP, ForeignFunctionInterface #-}
 
 -- |
@@ -76,7 +77,7 @@ import Data.Maybe ( isJust )
 
 import Foreign.C.String ( CString, withCString, peekCString )
 import Foreign.C.Error ( throwErrno )
-import Foreign.C.Types ( CInt )
+import Foreign.C.Types ( CInt(..) )
 
 #ifdef FORCE_CHAR8_ENCODING
 import GHC.IO.Encoding ( setFileSystemEncoding, setForeignEncoding, char8 )
