@@ -15,9 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# OPTIONS_GHC -cpp -fglasgow-exts #-}
-{-# LANGUAGE CPP #-}
--- , MagicHash, GADTs #-}
+{-# LANGUAGE CPP, FlexibleInstances #-}
 
 #include "gadts.h"
 
@@ -28,9 +26,10 @@ module Darcs.Witnesses.Sealed ( Sealed(..), seal, unseal, mapSeal,
                       unsealM, liftSM,
                       Gap(..), FreeLeft, unFreeLeft, FreeRight, unFreeRight
                     ) where
-
-import GHC.Base ( unsafeCoerce# )
+import Darcs.Witnesses.Eq ( MyEq, EqCheck(..) )
 import Darcs.Witnesses.Show
+import Darcs.Witnesses.Eq ( (=\/=) )
+import Darcs.Witnesses.Unsafe ( unsafeCoerceP1, unsafeCoerceP )
 
 data Sealed a where
     Sealed :: a C(x ) -> Sealed a
@@ -38,6 +37,10 @@ data Sealed a where
 seal :: a C(x ) -> Sealed a
 seal = Sealed
 
+instance MyEq a => Eq (Sealed (a C(x ))) where
+    Sealed x == Sealed y | IsEq <- x =\/= y = True
+                         | otherwise = False
+
 data Sealed2 a where
     Sealed2 :: !(a C(x y )) -> Sealed2 a
 
@@ -51,13 +54,13 @@ flipSeal :: a C(x y) -> FlippedSeal a C(y)
 flipSeal = FlippedSeal
 
 unsafeUnseal :: Sealed a -> a C(x)
-unsafeUnseal (Sealed a) = unsafeCoerce# a
+unsafeUnseal (Sealed a) = unsafeCoerceP1 a
 
 unsafeUnsealFlipped :: FlippedSeal a C(y) -> a C(x y)
-unsafeUnsealFlipped (FlippedSeal a) = unsafeCoerce# a
+unsafeUnsealFlipped (FlippedSeal a) = unsafeCoerceP a
 
 unsafeUnseal2 :: Sealed2 a -> a C(x y)
-unsafeUnseal2 (Sealed2 a) = unsafeCoerce# a
+unsafeUnseal2 (Sealed2 a) = unsafeCoerceP a
 
 unseal :: (FORALL(x) a C(x ) -> b) -> Sealed a -> b
 unseal f x = f (unsafeUnseal x)
@@ -99,7 +102,7 @@ newtype Poly a = Poly { unPoly :: FORALL(x) a C(x) }
 
 -- |'Stepped' is a type level composition operator.
 -- For example, 'Stepped Sealed p' is equivalent to 'lambda x . Sealed (p x)'
-newtype Stepped f a C(x) = Stepped { unStepped :: f (a C(x)) }
+newtype Stepped (f :: SEALEDPATCHKIND -> *) a C(x) = Stepped { unStepped :: f (a C(x)) }
 
 -- |'FreeLeft p' is '\forall x . \exists y . p x y'
 -- In other words the caller is free to specify the left witness,
@@ -133,7 +136,8 @@ class Gap w where
   -- |A 'Gap' constructed from a completely polymorphic value, for example the constructors
   -- for primitive patches
   freeGap :: (FORALL(x y) p C(x y)) -> w p
-  -- |Compose two 'Gap' values together, e.g. 'joinGap (+>+)' or 'joinGap (:>:)'
+
+  -- |Compose two 'Gap' values together in series, e.g. 'joinGap (+>+)' or 'joinGap (:>:)'
   joinGap :: (FORALL(x y z) p C(x y) -> q C(y z) -> r C(x z)) -> w p -> w q -> w r
 
 instance Gap FreeLeft where
@@ -146,4 +150,4 @@ instance Gap FreeRight where
   emptyGap e = FRInternal (Poly (FlippedSeal e))
   freeGap e =  FRInternal (Poly (FlippedSeal e))
   joinGap op (FRInternal p) (FRInternal q)
-    = FRInternal (Poly (case unPoly q of FlippedSeal q' -> case unPoly p of FlippedSeal p' -> FlippedSeal (p' `op` q')))
\ No newline at end of file
+    = FRInternal (Poly (case unPoly q of FlippedSeal q' -> case unPoly p of FlippedSeal p' -> FlippedSeal (p' `op` q')))
