@@ -15,19 +15,25 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE CPP, ForeignFunctionInterface, DeriveDataTypeable #-}
 
-{-# OPTIONS_GHC -fglasgow-exts #-}
-{-# LANGUAGE CPP, ForeignFunctionInterface #-}
--- , DeriveDataTypeable #-}
+-- |
+-- Module      : Exec
+-- Copyright   : 2003 David Roundy
+-- License     : GPL
+-- Maintainer  : darcs-devel@darcs.net
+-- Stability   : experimental
+-- Portability : portable
 
-module Exec ( exec, execInteractive,
-              withoutNonBlock,
-              Redirects, Redirect(..),
-              ExecException(..)
+module Exec (
+              exec
+            , execInteractive
+            , withoutNonBlock
+            , Redirects
+            , Redirect(..)
+            , ExecException(..)
             ) where
 
-import Data.Typeable ( Typeable, cast )
-
 #ifndef WIN32
 import Control.Exception.Extensible ( bracket )
 import System.Posix.Env ( setEnv, getEnv, unsetEnv )
@@ -38,13 +44,19 @@ import Control.Exception.Extensible ( catchJust, IOException )
 import Data.List ( isInfixOf )
 #endif
 
-import System.Exit ( ExitCode (..) )
+#if __GLASGOW_HASKELL__ >= 612
+import GHC.IO.Handle ( hDuplicate )
+#else
+import GHC.Handle ( hDuplicate )
+#endif
+
+import Control.Exception.Extensible ( bracketOnError, Exception(..),
+                                      SomeException(..) )
+import Data.Typeable ( Typeable, cast )
 import System.Cmd ( system )
+import System.Exit ( ExitCode (..) )
 import System.IO ( IOMode(..), openBinaryFile, stdout )
 import System.Process   ( runProcess, terminateProcess, waitForProcess )
-import GHC.Handle ( hDuplicate )
-        -- urgh.  hDuplicate isn't available from a standard place.
-import Control.Exception.Extensible ( bracketOnError, Exception(..), SomeException(..) )
 
 import Darcs.Global ( whenDebugMode )
 import Progress ( withoutProgress )
@@ -68,7 +80,10 @@ import Progress ( withoutProgress )
 -}
 
 type Redirects = (Redirect, Redirect, Redirect)
-data Redirect = AsIs | Null | File FilePath
+
+data Redirect = AsIs
+              | Null
+              | File FilePath
               | Stdout
                 deriving Show
 
@@ -78,12 +93,16 @@ data Redirect = AsIs | Null | File FilePath
   doesn't exist.
 -}
 --                   ExecException cmd    args     redirecs  errorDesc
-data ExecException = ExecException String [String] Redirects String
+data ExecException = ExecException
+                        String     -- cmd
+                        [String]   -- args
+                        Redirects  -- redirects
+                        String     -- errorDesc
                      deriving (Typeable,Show)
 
 instance Exception ExecException where
-   toException e = SomeException e
-   fromException (SomeException e) = cast e
+    toException e = SomeException e
+    fromException (SomeException e) = cast e
 
 _devNull :: FilePath
 #ifdef WIN32
@@ -96,29 +115,28 @@ _devNull = "/dev/null"
   We use System.Process, which does the necessary quoting
   and redirection for us behind the scenes.
 -}
-
 exec  :: String -> [String] -> Redirects -> IO ExitCode
 exec cmd args (inp,out,err) = withoutProgress $ do
-  h_stdin  <- redirect inp ReadMode
-  h_stdout <- redirect out WriteMode
-  h_stderr <- redirect err WriteMode
---  putStrLn (unwords (cmd:args ++ map show [inp,out,err]))
-  withExit127 $ bracketOnError
-    (do whenDebugMode $ putStrLn $ unwords $ cmd:args ++ ["; #"] ++ map show [inp,out,err]
-        runProcess cmd args Nothing Nothing h_stdin h_stdout h_stderr)
-    (terminateProcess)
-    (waitForProcess)
+    h_stdin  <- redirect inp ReadMode
+    h_stdout <- redirect out WriteMode
+    h_stderr <- redirect err WriteMode
+    withExit127 $ bracketOnError
+      (do doOptionalDebug
+          runProcess cmd args Nothing Nothing h_stdin h_stdout h_stderr)
+      (terminateProcess)
+      (waitForProcess)
   where
+    doOptionalDebug = whenDebugMode . putStrLn . unwords $
+        cmd : args ++ ["; #"] ++ map show [inp, out, err]
     redirect AsIs               _    = return Nothing
     redirect Null               mode = Just `fmap` openBinaryFile _devNull mode
     redirect (File "/dev/null") mode = redirect Null mode
     redirect (File f)           mode = Just `fmap` openBinaryFile f mode
+    -- hDuplicate stdout rather than passing stdout itself,
+    -- because runProcess closes the Handles we pass it.
     redirect Stdout             _    = Just `fmap` hDuplicate stdout
-        -- hDuplicate stdout rather than passing stdout itself,
-        -- because runProcess closes the Handles we pass it.
 
 execInteractive :: String -> String -> IO ExitCode
-
 #ifndef WIN32
 {-
 This should handle arbitrary commands interpreted by the shell on Unix since
@@ -127,26 +145,40 @@ the argument in any way, so we set an environment variable and call
 cmd "$DARCS_ARGUMENT"
 -}
 execInteractive cmd arg = withoutProgress $ do
-  let var = "DARCS_ARGUMENT"
-  stdin `seq` return ()
-  withoutNonBlock $ bracket
-    (do oldval <- getEnv var
-        setEnv var arg True
-        return oldval)
-    (\oldval ->
-       do case oldval of
-            Nothing -> unsetEnv var
-            Just val -> setEnv var val True)
-    (\_ -> withExit127 $ system $ cmd++" \"$"++var++"\"")
+    let var = "DARCS_ARGUMENT"
+    stdin `seq` return ()
+    withoutNonBlock $ bracket
+      (do oldval <- getEnv var
+          setEnv var arg True
+          return oldval)
+      (\oldval ->
+         case oldval of
+              Nothing -> unsetEnv var
+              Just val -> setEnv var val True)
+      (\_ -> withExit127 $ system $ cmd++" \"$"++var++"\"")
 
 #else
-
+-- The `system' function passes commands to execute via cmd.exe (or
+-- command.com) it's return value is equivalent to the one returned by the
+-- shell. For regular applications - this works correctly resulting in the
+-- exit code of the program. However in case of a command/file which can't be
+-- found - cmd.exe will return 1 instead of propagating the ExitFailure 9009
+-- which on windows is equivalent to ExitFailure 127 from *nix machines.
+--
+-- Here we force return the exit code of the last cmd.exe action by appending
+-- & exit !errorlevel! to the command being executed that way chaining with
+-- ortryrunning works correctly.
+--
+-- SETLOCAL EnableDelayedExpansion makes sure that !variable! expansion is done
+-- correctly on systems where that function is not enabled by default.
+--
 execInteractive cmd arg = withoutProgress $ do
-  system $ cmd ++ " " ++ arg
+  withExit127 $ system $ "SETLOCAL EnableDelayedExpansion & " ++
+                          cmd ++ " " ++ arg ++
+                          " & exit !errorlevel!"
 #endif
 
 withoutNonBlock :: IO a -> IO a
-
 #ifndef WIN32
 {-
 Do IO without NonBlockingRead on stdInput.
@@ -159,10 +191,10 @@ withoutNonBlock x =
     do nb <- queryFdOption stdInput NonBlockingRead
        if nb
           then bracket
-                   (do setFdOption stdInput NonBlockingRead False)
+                   (setFdOption stdInput NonBlockingRead False)
                    (\_ -> setFdOption stdInput NonBlockingRead True)
                    (\_ -> x)
-          else do x
+          else x
 #else
 withoutNonBlock x = do x
 #endif
