@@ -1,14 +1,22 @@
 {-# LANGUAGE CPP, ForeignFunctionInterface #-}
 
+-- |
+-- Module      : URL
+-- Copyright   : 2008 Dmitry Kurochkin <dmitry.kurochkin@gmail.com>
+-- License     : GPL
+-- Maintainer  : darcs-devel@darcs.net
+-- Stability   : experimental
+-- Portability : portable
+
 module URL ( copyUrl, copyUrlFirst, setDebugHTTP,
              disableHTTPPipelining, maxPipelineLength,
              waitUrl, Cachable(Cachable, Uncachable, MaxAge),
-             environmentHelpProxy, environmentHelpProxyPassword
+             environmentHelpProxy, environmentHelpProxyPassword,
+             ConnectionError(..),
            ) where
 
 import Data.IORef ( newIORef, readIORef, writeIORef, IORef )
 import Data.Map ( Map )
-import Data.List ( delete )
 import qualified Data.Map as Map
 import System.Directory ( copyFile )
 import System.IO.Unsafe ( unsafePerformIO )
@@ -18,66 +26,26 @@ import Control.Concurrent.MVar ( isEmptyMVar, modifyMVar_, newEmptyMVar, newMVar
 import Control.Monad ( unless, when )
 import Control.Monad.Trans ( liftIO )
 import Control.Monad.State ( evalStateT, get, modify, put, StateT )
-import Foreign.C.Types ( CInt )
 
 import Workaround ( renameFile )
 import Darcs.Global ( atexit )
-import Progress ( debugFail, debugMessage )
+import Progress ( debugMessage )
 import Darcs.Lock ( removeFileMayNotExist )
 
 import Numeric ( showHex )
 import System.Random ( randomRIO )
+import URL.Request
 
 #ifdef HAVE_CURL
-import Foreign.C.String ( withCString, peekCString, CString )
+import qualified URL.Curl as Curl
+#elif defined(HAVE_HTTP)
+import qualified URL.HTTP as HTTP 
 #else
+import Progress ( debugFail )
 import qualified HTTP ( requestUrl, waitNextUrl )
 #endif
 #include "impossible.h"
 
-data UrlRequest = UrlRequest { url :: String
-                             , file :: FilePath
-                             , cachable :: Cachable
-                             , priority :: Priority }
-
-data Cachable = Cachable | Uncachable | MaxAge !CInt
-                deriving (Show, Eq)
-
-data UrlState = UrlState { inProgress :: Map String ( FilePath
-                                                    , [FilePath]
-                                                    , Cachable )
-                         , waitToStart :: Q String
-                         , pipeLength :: Int
-                         , randomJunk :: String }
-
-data Q a = Q [a] [a]
-
-readQ :: Q a -> Maybe (a, Q a)
-readQ (Q (x:xs) ys) = Just (x, Q xs ys)
-readQ (Q [] ys) = do x:xs <- Just $ reverse ys
-                     Just (x, Q xs [])
-
-insertQ :: a -> Q a -> Q a
-insertQ y (Q xs ys) = Q xs (y:ys)
-
-pushQ :: a -> Q a -> Q a
-pushQ x (Q xs ys) = Q (x:xs) ys
-
-deleteQ :: Eq a => a -> Q a -> Q a
-deleteQ x (Q xs ys) = Q (delete x xs) (delete x ys)
-
-elemQ :: Eq a => a -> Q a -> Bool
-elemQ x (Q xs ys) = x `elem` xs || x `elem` ys
-
-emptyQ :: Q a
-emptyQ = Q [] []
-
-nullQ :: Q a -> Bool
-nullQ (Q [] []) = True
-nullQ _         = False
-
-data Priority = High | Low deriving Eq
-
 {-# NOINLINE maxPipelineLengthRef #-}
 maxPipelineLengthRef :: IORef Int
 maxPipelineLengthRef = unsafePerformIO $ do
@@ -100,42 +68,60 @@ urlNotifications = unsafePerformIO $ newMVar Map.empty
 urlChan :: Chan UrlRequest
 urlChan = unsafePerformIO $ do
   ch <- newChan
-  forkIO (urlThread ch)
+-- Warning:  A do-notation statement discarded a result of type GHC.Conc.ThreadId.
+  _ <- forkIO (urlThread ch)
   return ch
 
+-- ----------------------------------------------------------------------
+-- urlThread
+-- ----------------------------------------------------------------------
+
+type UrlM a = StateT UrlState IO a
+
 urlThread :: Chan UrlRequest -> IO ()
 urlThread ch = do junk <- flip showHex "" `fmap` randomRIO rrange
-                  evalStateT urlThread' (UrlState Map.empty emptyQ 0 junk)
+                  evalStateT (urlThread' ch) (UrlState Map.empty emptyQ 0 junk)
     where rrange = (0, 2^(128 :: Integer) :: Integer)
-          urlThread' = do
+
+-- | Internal to urlThread
+urlThread' :: Chan UrlRequest -> UrlM ()
+urlThread' ch = do
             empty <- liftIO $ isEmptyChan ch
             st <- get
             let l = pipeLength st
                 w = waitToStart st
             reqs <- if not empty || (nullQ w && l == 0)
-                    then liftIO readAllRequests
+                    then liftIO (readAllRequests ch)
                     else return []
             mapM_ addReq reqs
             checkWaitToStart
             waitNextUrl
-            urlThread'
-          readAllRequests = do
+            urlThread' ch
+
+-- | Internal to urlThread
+readAllRequests :: Chan UrlRequest -> IO [UrlRequest]
+readAllRequests ch = do
             r <- readChan ch
             debugMessage $ "URL.urlThread ("++url r++"\n"++
                            "            -> "++file r++")"
             empty <- isEmptyChan ch
             reqs <- if not empty
-                    then readAllRequests
+                    then readAllRequests ch
                     else return []
             return (r:reqs)
-          addReq r = do
-            let u = url r
-                f = file r
-                c = cachable r
-            d <- liftIO $ alreadyDownloaded u
-            if d
-               then dbg "Ignoring UrlRequest of URL that is already downloaded."
-               else do
+
+-- | Internal to urlThread
+addReq :: UrlRequest -> UrlM ()
+addReq r = do
+  d <- liftIO (alreadyDownloaded u)
+  if d
+     then dbg "Ignoring UrlRequest of URL that is already downloaded."
+     else reallyAdd
+ where
+   f = file r
+   c = cachable r
+   u = url r
+   reallyAdd = do
                  st <- get
                  let p = inProgress st
                      w = waitToStart st
@@ -160,12 +146,18 @@ urlThread ch = do junk <- flip showHex "" `fmap` randomRIO rrange
                                    dbg "Adding new file to existing UrlRequest."
                         else dbg "Ignoring UrlRequest of file that's already queued."
                    _ -> put new_st
-          alreadyDownloaded u = do
-            n <- liftIO $ withMVar urlNotifications (return . (Map.lookup u))
+
+
+-- | Internal to urlThread
+alreadyDownloaded :: String -> IO Bool
+alreadyDownloaded u = do
+            n <- withMVar urlNotifications (return . (Map.lookup u))
             case n of
               Just v  -> not `fmap` isEmptyMVar v
               Nothing -> return True
 
+-- ----------------------------------------------------------------------
+
 checkWaitToStart :: StateT UrlState IO ()
 checkWaitToStart = do
   st <- get
@@ -182,8 +174,7 @@ checkWaitToStart = do
             let f_new = f++"-new_"++randomJunk st
             err <- liftIO $ requestUrl u f_new c
             if null err
-               then do dbg "URL.requestUrl succeeded"
-                       liftIO $ atexit (removeFileMayNotExist f_new)
+               then do liftIO $ atexit (removeFileMayNotExist f_new)
                        put $ st { waitToStart = rest
                                 , pipeLength = l + 1 }
                else do dbg $ "Failed to start download URL "++u++": "++err
@@ -216,7 +207,7 @@ waitNextUrl = do
   let l = pipeLength st
   when (l > 0) $ do
                 dbg "URL.waitNextUrl start"
-                (u, e) <- liftIO $ waitNextUrl'
+                (u, e, ce) <- liftIO $ waitNextUrl'
                 let p = inProgress st
                     new_st = st { inProgress = Map.delete u p
                                 , pipeLength = l - 1 }
@@ -231,7 +222,9 @@ waitNextUrl = do
                          else case Map.lookup u p of
                                 Just (f, _, _) -> do
                                   removeFileMayNotExist (f++"-new_"++randomJunk st)
-                                  downloadComplete u e
+                                  case ce of
+                                    Just httpError -> downloadComplete u (show httpError)
+                                    Nothing        -> downloadComplete u e
                                   debugMessage $ "URL.waitNextUrl failed: "++
                                                u++" "++f++" "++e
                                 Nothing -> bug $ "Another possible bug in URL.waitNextUrl: "++u++" "++e
@@ -254,7 +247,9 @@ waitUrl u = do debugMessage $ "URL.waitUrl "++u
                  Just var -> do
                         e <- readMVar var
                         modifyMVar_ urlNotifications (return . (Map.delete u))
-                        unless (null e) (debugFail $ "Failed to download URL "++u++": "++e)
+                        unless (null e) $ do
+                          debugMessage $ "Failed to download URL "++u++": "++e
+                          fail e
                  Nothing  -> return () -- file was already downloaded
 
 dbg :: String -> StateT a IO ()
@@ -268,54 +263,20 @@ minCachable (MaxAge a) _          = MaxAge a
 minCachable _          (MaxAge b) = MaxAge b
 minCachable _          _          = Cachable
 
-#ifdef HAVE_CURL
-cachableToInt :: Cachable -> CInt
-cachableToInt Cachable = -1
-cachableToInt Uncachable = 0
-cachableToInt (MaxAge n) = n
-#endif
-
 disableHTTPPipelining :: IO ()
 disableHTTPPipelining = writeIORef maxPipelineLengthRef 1
 
 setDebugHTTP :: IO ()
 requestUrl :: String -> FilePath -> Cachable -> IO String
-waitNextUrl' :: IO (String, String)
+waitNextUrl' :: IO (String, String, Maybe ConnectionError)
 pipeliningEnabled :: IO Bool
 
 #ifdef HAVE_CURL
 
-setDebugHTTP = curl_enable_debug
-
-requestUrl u f cache =
-    withCString u $ \ustr ->
-    withCString f $ \fstr -> do
-      err <- curl_request_url ustr fstr (cachableToInt cache) >>= peekCString
-      return err
-
-waitNextUrl' = do
-  e <- curl_wait_next_url >>= peekCString
-  u <- curl_last_url >>= peekCString
-  return (u, e)
-
-pipeliningEnabled = do
-  r <- curl_pipelining_enabled
-  return $ r /= 0
-
-foreign import ccall "hscurl.h curl_request_url"
-  curl_request_url :: CString -> CString -> CInt -> IO CString
-
-foreign import ccall "hscurl.h curl_wait_next_url"
-  curl_wait_next_url :: IO CString
-
-foreign import ccall "hscurl.h curl_last_url"
-  curl_last_url :: IO CString
-
-foreign import ccall "hscurl.h curl_enable_debug"
-  curl_enable_debug :: IO ()
-
-foreign import ccall "hscurl.h curl_pipelining_enabled"
-  curl_pipelining_enabled :: IO CInt
+setDebugHTTP = Curl.setDebugHTTP
+requestUrl = Curl.requestUrl
+waitNextUrl' = Curl.waitNextUrl
+pipeliningEnabled = Curl.pipeliningEnabled
 
 #elif defined(HAVE_HTTP)
 
