@@ -25,11 +25,14 @@ module Workaround
     , sigPIPE
     ) where
 
+import Prelude hiding ( catch )
+
 #ifdef WIN32
 
 import qualified System.Directory ( renameFile, getCurrentDirectory, removeFile )
+import Control.Exception ( catch, IOException )
 import qualified Control.Exception ( block )
-import qualified System.IO.Error ( isDoesNotExistError, ioError, catch )
+import qualified System.IO.Error ( isDoesNotExistError, ioError )
 
 #else
 
@@ -95,9 +98,9 @@ renameFile :: FilePath
            -> IO ()
 renameFile old new = Control.Exception.block $
    System.Directory.renameFile old new
-   `System.IO.Error.catch` \_ ->
+   `catch` \(_ :: IOException) ->
    do System.Directory.removeFile new
-        `System.IO.Error.catch`
+        `catch`
          (\e -> if System.IO.Error.isDoesNotExistError e
                    then return ()
                    else System.IO.Error.ioError e)
