@@ -250,8 +250,11 @@ const char *curl_request_url(const char *url,
   return error_strings[RESULT_OK];
 }
 
-const char *curl_wait_next_url()
+const char *curl_wait_next_url(int* errorCode, long* httpErrorCode)
 {
+  *errorCode = -1;
+  *httpErrorCode = -1;
+
   if (last_url != NULL)
     {
       free(last_url);
@@ -274,9 +277,18 @@ const char *curl_wait_next_url()
       CURL *easy = msg->easy_handle;
       CURLcode result = msg->data.result;
       struct UrlData *url_data;
-      int error = curl_easy_getinfo(easy, CURLINFO_PRIVATE, (char **)&url_data);
-      if (error != CURLE_OK)
+
+      int error = set_time_out(easy, errorCode);
+      if (error != CURLE_OK ){
+        *errorCode = error;
         return curl_easy_strerror(error);
+      }
+
+      error = curl_easy_getinfo(easy, CURLINFO_PRIVATE, (char **)&url_data);
+      if (error != CURLE_OK){
+        *errorCode = error;
+        return curl_easy_strerror(error);
+      }
 
       last_url = url_data->url;
       fclose(url_data->file);
@@ -288,8 +300,14 @@ const char *curl_wait_next_url()
         return curl_multi_strerror(error);
       curl_easy_cleanup(easy);
 
-      if (result != CURLE_OK)
+      if (result != CURLE_OK){
+        *errorCode = result;
+
+        if (result == CURLE_HTTP_RETURNED_ERROR)
+		  curl_easy_getinfo(easy, CURLINFO_RESPONSE_CODE, httpErrorCode);
+
         return curl_easy_strerror(result);
+      }
     }
   else
     return error_strings[RESULT_UNKNOWN_MESSAGE];
@@ -315,3 +333,23 @@ int curl_pipelining_enabled()
   return 0;
 #endif
 }
+
+int set_time_out(CURL *handle, int* errorCode)
+{
+  int error;
+  long time_out = DEFAULT_CONNECTION_TIMEOUT;
+  const char *stime_out;
+
+  stime_out = getenv("DARCS_CONNECTION_TIMEOUT");
+  if (stime_out != NULL){
+    long result = atol (stime_out);
+    if ( result > 0 )
+      time_out = result;
+    else
+      *errorCode = 90 ;
+  }
+
+  error = curl_easy_setopt(handle, CURLOPT_TIMEOUT, time_out);
+
+  return error;
+}
