@@ -16,25 +16,21 @@ mkdir foo.d
 mkdir oof.d
 darcs add foo.d
 darcs add oof.d
-not darcs add foo.d 2> log
-grep -i directory log
+not darcs add foo.d 2>&1 | grep -i directory
 # Try adding the same directory when it's already in the repo
-not darcs add foo.d oof.d 2> log
-grep -i directories log
+not darcs add foo.d oof.d 2>&1 | grep -i directories
 
 # Make sure that messages about files call them files
 touch bar
 touch baz
 darcs add bar
 darcs add baz
-not darcs add bar 2> log
-grep -i 'following file is' log
-not darcs add bar baz 2> log
-grep -i 'following files are' log
+not darcs add bar 2>&1 | grep -i "following file is"
+not darcs add bar baz 2>&1 | grep -i "following files are"
 
 # Make sure that messages about both files and directories say so
-not darcs add bar foo.d 2> log
-grep -i 'files and directories' log
+not darcs add bar foo.d 2>&1 | grep -i 'files and directories'
+
 
 # Make sure that parent directories are added for files
 mkdir -p a.d/aa.d/aaa.d
@@ -56,8 +52,7 @@ touch d.d/foo
 darcs add -rv d.d | grep 'd.d/foo'
 
 # 'adding a non-existent dir and file gives the expected message
-not darcs add notadir/notafile 2> log
-grep -i 'does not exist' log
+not darcs add notadir/notafile 2>&1 | grep -i 'does not exist'
 
 cd ..
 rm -rf temp1
