@@ -1,34 +1,31 @@
-#! /bin/sh
+#!/usr/bin/env bash
 
 set -ev
 
-test $DARCS || DARCS=`pwd`/../darcs
-darcs=$DARCS
-
 rm -rf tmp
 mkdir tmp
 cd tmp
-$darcs init
+darcs init
 
 empty='test ! -s'
 nonempty='test -s'
 
 rm -f foo
-$darcs add foo >stdout 2>stderr && exit 1 || true
+darcs add foo >stdout 2>stderr && exit 1 || true
 $empty stdout
 $nonempty stderr
 
 >foo
-$darcs add foo >stdout 2>stderr
+darcs add foo >stdout 2>stderr
 $empty stdout
 $empty stderr
 
-$darcs add foo >stdout 2>stderr && exit 1 || true
+darcs add foo >stdout 2>stderr && exit 1 || true
 $empty stdout
 $nonempty stderr
 
 rm foo
-$darcs add foo >stdout 2>stderr && exit 1 || true
+darcs add foo >stdout 2>stderr && exit 1 || true
 $empty stdout
 $nonempty stderr
 
