@@ -1,19 +1,16 @@
 #!/usr/bin/env bash
-set -ev
+. lib
 
-rm -rf temp
-mkdir temp
-cd temp
 darcs init
 mkdir a b
 touch a/a b/b
 darcs add --rec .
 darcs record -a -m ab -A test
 darcs annotate a/a
-# annotate --xml should encode angle brackets in user name
-touch c
+echo x > c
 darcs add c
 darcs record -a -m foo -A 'Mark Stosberg <a@b.com>'
-darcs annotate --xml c | grep "&lt;a\@b.com&gt;"
+darcs annotate c
+darcs annotate c | grep "a@b.com"
 cd ..
 rm -rf temp
