@@ -12,11 +12,11 @@ echo abc > A
 darcs add A
 echo def > B1
 darcs add B1
-# darcs record -all --patch-name patch1 # this way it doesn't trigger the bug
-for i in 1 2 3 4 5 6 7 8 9 11; do echo y; done | darcs record --patch-name patch1
+# darcs record -all --name patch1 # this way it doesn't trigger the bug
+for i in 1 2 3 4 5 6 7 8 9 11; do echo y; done | darcs record --name patch1
 
 darcs mv B1 B2
-darcs record --all --patch-name patch2
+darcs record --all --name patch2
 cd ..
 
 mkdir temp2
