@@ -1,13 +1,15 @@
 #!/usr/bin/env bash
 
-. ../tests/lib
+. lib
 
 set -ev
 switch_to_latin9_locale
 rm -rf temp1
 rm -rf temp2
+rm -rf temp3
 mkdir temp1
 mkdir temp2
+mkdir temp3
 cd temp1
 
 seventysevenaddy="<aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa@bbbbbbbbbb.cccccccccc.abrasoft.com>"
@@ -34,10 +36,17 @@ cat mail_as_file
 grep $seventysevenaddy mail_as_file
 
 # Check that there are no non-ASCII characters in the mail
-ghc -e 'getContents >>= return . not . any (> Data.Char.chr 127)' < mail_as_file | grep '^True$'
+cd ../temp3
+cat > is_ascii.hs <<EOF
+import Data.Char (chr)
 
+main = getContents >>= print . not . any (> Data.Char.chr 127)
+EOF
+
+ghc --make is_ascii.hs -o is_ascii
+./is_ascii < ../temp1/mail_as_file | grep '^True$'
 
 cd ..
 rm -rf temp1
 rm -rf temp2
-
+rm -rf temp3
