@@ -25,7 +25,7 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-. ../tests/lib
+. lib
 set -ev
 rm -rf R
 mkdir R
