@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 set -ev
 
-. ../tests/lib
+. lib
 
 rm -rf temp1 temp1a temp1b
 
