@@ -60,7 +60,7 @@ darcs pull --all ../temp2
 # we should have a marked conflict now.
 grep 'v v' A
 # we resolve it simply by removing conflict markers.
-sed -e '/\^ \^|\*\*|v v/d' A > temp
+sed -e '/\^ \^\|\*\*\|v v/d' A > temp
 mv temp A
 darcs record -m resolution --all
 # now mark-conflicts shouldn't find any unmarked conflicts
