@@ -1,11 +1,11 @@
 #!/usr/bin/env bash
 set -ev
-rm -rf test
-mkdir test
-cd test
+rm -rf temp1
+mkdir temp1
+cd temp1
 darcs --version
 darcs init
-echo test >File.hs
+echo temp1 >File.hs
 darcs add File.hs
 darcs record File.hs -a -m "add File"
 rm File.hs
@@ -14,3 +14,5 @@ darcs cha
 darcs unrecord -p "rm File" -a
 darcs cha
 darcs record -a -m "re-rm File"
+cd ..
+rm -rf temp1
