@@ -11,7 +11,7 @@ cd temp1
 darcs init
 echo a > a
 darcs add a
-darcs record --all --patch-name=a
+darcs record --all --name=a
 cd ../temp2
 darcs init
 darcs pull --all --dont-allow-conflicts ../temp1
@@ -28,7 +28,7 @@ cd ../temp2
 darcs init
 echo a > a
 darcs add a
-darcs record --all --patch-name=a
+darcs record --all --name=a
 darcs push --all ../temp1
 cd ../temp1
 test `darcs changes --count` = "1"
@@ -42,7 +42,7 @@ cd temp1
 darcs init
 echo a > a
 darcs add a
-darcs record --all --patch-name=a
+darcs record --all --name=a
 
 for i in 1 2 3 4 5 6 7 8 9; do
     echo Change number $i >> a
@@ -53,7 +53,7 @@ cd ../temp2
 darcs init
 echo b > b
 darcs add b
-darcs record --all --patch-name=b
+darcs record --all --name=b
 
 for i in 1 2 3 4 5 6 7 8 9; do
     echo Change number $i >> b
