@@ -1,8 +1,6 @@
 #!/usr/bin/env bash
-set -ev
 
-## The builtin ! has the wrong semantics for not.
-not () { "$@" && exit 1 || :; }
+. lib
 
 rm -rf temp1 temp2
 
