@@ -12,12 +12,12 @@ darcs record -am 'add foo'
 echo newtext > foo
 darcs wh
 
-darcs diff --store > out1
+darcs diff --no-unified --store > out1
 cat out1
 grep text out1
 grep foo out1
 
-darcs diff > out
+darcs diff --no-unified > out
 cat out
 grep text out
 grep foo out
