@@ -5,17 +5,8 @@
 ## Placed into the public domain by Ganesh Sittampalam, 2009
 
 . lib
-rm -rf temp1
-mkdir temp1
-cd temp1
-darcs init --old-fashioned
-touch foo
-darcs rec -a -m'foo' -l --author=me@me
-darcs tag --author=me@me foo
+
+gunzip -c  $TESTDATA/oldfashioned-compressed.tgz | tar xf -
+cd oldfashioned-compressed
+darcs optimize --upgrade
 darcs check
-darcs optimize --reorder
-darcs check
-darcs optimize --compress
-darcs check
-cd ..
-rm -rf temp1
