@@ -25,14 +25,14 @@
 ## SOFTWARE.
 
 set -ev
-. ../tests/lib
+. lib
 rm -rf R; mkdir R; cd R
 
 darcs init
 darcs setpref test false
 darcs record  -am foo --no-test
 export DARCS_EDITOR="echo 'new log' > "
-echo y | not darcs amend -p foo --edit-long-comment --test 2> out
+echo yn | not darcs amend -p foo --edit-long-comment --test 2> out
 
 # the msg has the format: "Logfile left in filenamehere."
 LOGFILE=`grep "Logfile left in" out | sed "s/Logfile left in //" | sed s/.$//`
