@@ -25,12 +25,12 @@
 ## SOFTWARE.
 
 set -ev
-. ../tests/lib
+. lib
 rm -rf R; mkdir R; cd R
 export DARCS_EDITOR="echo 'a log' > "
 darcs init
 darcs setpref test false
-echo yy | not darcs record -m foo -a --edit-long-comment --test 2> out
+echo yy| not darcs record -m foo -a --edit-long-comment --test 2> out
 
 # the msg has the format: "Logfile left in filenamehere."
 LOGFILE=`grep "Logfile left in" out | sed "s/Logfile left in //" | sed s/.$//`
