@@ -8,7 +8,7 @@ cd temp2/ ; echo 'x' > _darcs/prefs/author ; cd ..
 cd temp1/ ; echo 'x' > _darcs/prefs/author ; cd ..
 
 cd temp1/
-touch test ; darcs record
+touch test
 darcs add test     ; darcs record -a -m 'test'
 darcs mv test best ; darcs record -a -m 'test -> best'
 darcs mv best test ; darcs record -a -m 'best -> test'
