@@ -35,7 +35,7 @@ EOF
 
 chmod +x editor
 
-DARCS_EDITOR='bash editor' darcs send --debug --author=me -a --output=bundle --edit-description ../temp2
+DARCS_EDITOR='bash editor' darcs send --author=me -a --output=bundle --edit-description ../temp2
 
 echo === beginning of bundle > ===
 cat bundle
