@@ -20,7 +20,8 @@ printf >repo1/x '#!/bin/sh\ntrue'   # make a shebang'd script
 darcs record --repodir repo1 -lam x x
 umask 077                       # DENY ALL access to group, all
 darcs get --set-scripts-executable repo1 repo2
-ls -l repo2/x | cut -f 1 -d\  > mode
+# remove trailing-dot for xattr
+ls -l repo2/x | cut -f 1 -d\  | sed -e "s/\.$//" > mode
 echo -rwx------ > desired-mode
 diff -u desired-mode mode
 
