@@ -23,7 +23,7 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-. ../tests/lib
+. lib
 
 rm -rf S T
 darcs get --lazy http://darcs.net/testing/repo1 S
