@@ -8,14 +8,14 @@ cd temp1
 darcs init
 echo first > a
 darcs add a
-darcs record --pipe --all --patch-name=first <<EOF
+darcs record --pipe --all --name=first <<EOF
 Thu Sep 18 22:37:06 MSD 2008
 author
 EOF
 
 echo first > b
 darcs add b
-darcs record --pipe --all --patch-name=first <<EOF
+darcs record --pipe --all --name=first <<EOF
 Thu Sep 18 22:37:06 MSD 2008
 author
 EOF
