@@ -34,7 +34,7 @@ mkdir z1
 echo "hi" > z1/foo
 mkdir z2
 if ! ln z1/foo z2/foo ; then
-  echo No ln command for `pwd`; assuming no hard links.
+  echo No ln command for `pwd` - assuming no hard links.
   exit 0
 fi
 if ! compare z1 z2 ; then
