@@ -7,7 +7,7 @@ cd temp
 darcs init
 date > bla
 darcs add bla
-darcs record -a --patch-name=11
+darcs record -a --name=11
 echo hello > world
 darcs add world
 darcs whatsnew --dont-look-for-adds > wn1
@@ -17,7 +17,7 @@ cd temp_0
 date > bla2
 date >> bla
 darcs add bla2
-darcs record -a --patch-name=22
+darcs record -a --name=22
 darcs push -a ../temp
 cd ..
 cd temp
