@@ -38,11 +38,9 @@ darcs revert -a
 
 write_buggy_pending
 
-darcs record -a -m foo 2>&1 | tee out
+not darcs record -a -m foo 2>&1 | tee out
 grep 'pending has conflicts' out
 
-darcs record -a -m foo
-
 darcs changes -v
 
 darcs repair 2>&1 | tee out
