@@ -22,10 +22,12 @@ darcs record -Ax -alm foo2
 echo bar2 > b
 darcs record -Ax -alm bar2
 cd ../temp2
-echo y | darcs pull ../temp1 --dont-prompt-for-dependencies -p foo2 --dry-run > toto
-#on the previous line, we don't get asked about foo1.
-grep foo toto | wc -l | grep 2
-#but we send it anyway.
+echo y | darcs pull ../temp1 -i --dont-prompt-for-dependencies -p foo2 --dry-run > toto
+#on the previous line, we get asked about foo2, and we take it
+grep foo2 toto | wc -l | grep 2
+#we don't get asked about foo1, but we take it anyway, so 
+grep foo1 toto | wc -l | grep 1
+#and we don't send bar
 grep bar toto | wc -l | grep 0
 cd ..
 rm -rf temp1 temp2
