@@ -1,10 +1,9 @@
 #!/usr/bin/env bash
 
-## The builtin ! has the wrong semantics for not.
-not () { "$@" && exit 1 || :; }
+. lib
+
 # This test script is in the public domain.
 
-set -ev
 
 rm -rf temp1 temp2 temp3
 
