@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. lib
 
 rm -rf temp1 temp2
 
@@ -11,7 +11,7 @@ else format=hashed; fi
 
 mkdir temp2
 cd temp2
-gunzip -c ../repos/many-files--${format}.tgz | tar xf -
+gunzip -c $TESTDATA/many-files--${format}.tgz | tar xf -
 cd ..
 
 mkdir temp1
