@@ -1,39 +1,34 @@
-#!/bin/sh
+#!/usr/bin/env bash
 
 # This test script, originally written by David Roundy is in the public
 # domain.
 
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
-ACTUAL_DARCS=`which $DARCS`
-DARCSPATH=`dirname $ACTUAL_DARCS`
-export PATH="$DARCSPATH:$PATH"
-
 rm -rf temp1 temp2
 
 mkdir temp1
 cd temp1
 echo foo > bar
-$DARCS initialize
+darcs initialize
 echo record author me > _darcs/prefs/defaults
-$DARCS add bar
-$DARCS record -a -m addbar
+darcs add bar
+darcs record -a -m addbar
 
 cd ..
-$DARCS get temp1 temp2
+darcs get temp1 temp2
 cd temp1
 date > bar
-$DARCS record -a -m datebar
+darcs record -a -m datebar
 
 cd ../temp1
 echo aack >> bar
-$DARCS record -a -m aackbar
+darcs record -a -m aackbar
 
 cd ../temp2
 
-$DARCS pull -av
-$DARCS check
+darcs pull -av
+darcs check
 
 cd ..
 rm -rf temp1 temp2
