@@ -29,6 +29,28 @@ echo y | darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DES
 cd ..
 rm -rf temp2\ dir
 
+# make sure summaries are coalesced
+mkdir temp3
+cd temp3
+darcs init
+cat > file <<EOF
+1
+2
+3
+4
+EOF
+darcs add file
+darcs rec -a -m "init" file
+cat > file <<EOF
+A
+2
+3
+B
+EOF
+echo y | darcs record --edit-long-comment -a -m edit | grep -c "./file" | grep 1
+cd ..
+rm -rf temp2\ dir
+
 export DARCS_EDITOR='grep "END OF"'
 # editor: quoting in command
 mkdir temp1
