@@ -8,10 +8,15 @@ cd temp
 darcs init
 
 echo "X X X" > foo
+echo $'A A,A\tA,A\vA' >> foo
 darcs rec -alm "Added"
 
-# This should fail until replace handles spaces
+# These should fail until replace handles tokens and
+# token-chars with leteral spaces in them
 darcs replace ' X ' ' XX ' --token-chars '[ X]' foo && exit 1 || true
+darcs replace $'A A'  'aaa' --token-chars '[^,]' foo && exit 1 || true
+darcs replace $'A\tA' 'aaa' --token-chars '[^,]' foo && exit 1 || true
+darcs replace $'A\vA' 'aaa' --token-chars '[^,]' foo && exit 1 || true
 
 # Check that replace is not fooled by duplicate file names
 # (i.e. not trying to performe the replace twice in the same file)
