@@ -10,7 +10,7 @@ darcs init
 
 echo a b a b a b > A
 darcs add A
-darcs record --all --patch-name=p1
+darcs record --all --name=p1
 darcs mv A B
 if darcs replace a c B | grep Skipping; then
     exit 1
