@@ -1,18 +1,16 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
-
 rm -rf temp1
 mkdir temp1
 cd temp1
-$DARCS init
+darcs init
 echo hello world > foo
-$DARCS add foo
-$DARCS record -a -m add -A x
+darcs add foo
+darcs record -a -m add -A x
 echo goodbye world >> foo
-echo y/y | tr / \\012 | $DARCS revert
-cmp foo _darcs/pristine/foo
+echo y/y | tr / \\012 | darcs revert
+darcs show contents foo | cmp foo -
 
 # Now let's test a trickier revert where changes commute nontrivially.
 
@@ -24,42 +22,45 @@ d
 e
 EOF
 
-$DARCS record -a -A me -m cleanup
+darcs record -a -A me -m cleanup
 
 mv foo foo.tmp
 cat foo.tmp | grep -v b | grep -v d > foo
 
-echo "nyy" | $DARCS revert
+echo "nyy" | darcs revert
 
-DARCS_DONT_COLOR=1 $DARCS wh > whatsnew
+DARCS_DONT_COLOR=1 darcs wh > whatsnew
 cat > correct <<EOF
-{
 hunk ./foo 2
 -b
-}
 EOF
-diff -u correct whatsnew
+diff -c correct whatsnew
 
 # Try a situation where earlier (kept) changes are depended upon by the
 # changes we want to revert:
 
-$DARCS record -a -A me -m cleanup
+darcs record -a -A me -m cleanup
 
 echo hello world > bar
-$DARCS add bar
-$DARCS replace hello goodbye bar
+echo hello world > foo
+darcs add bar
+darcs replace hello goodbye bar foo
 
-echo "nnyy/y" | tr / \\012 | $DARCS revert
+echo "cnnnyy/y" | tr / \\012 | darcs revert
 
-DARCS_DONT_COLOR=1 $DARCS wh > whatsnew
+DARCS_DONT_COLOR=1 darcs wh > whatsnew
 cat > correct <<EOF
-{
 addfile ./bar
 hunk ./bar 1
++goodbye world
+hunk ./foo 1
+-a
+-c
+-d
+-e
 +hello world
-}
 EOF
-diff -u correct whatsnew
+diff -c correct whatsnew
 
 cd ..
 rm -rf temp1
