@@ -23,11 +23,11 @@ do
   if darcs whatsnew ; then
       : Resolve conflict - rollback our patch
       darcs revert --all
-      darcs rollback -a --patch-name x0
+      darcs rollback -a --name x0
       darcs revert --all
   fi
   : Create local change and record it
-  echo X$i > f && darcs record -A me --ignore-time -m x$i --all f
+  echo X$i > f && darcs record -l -A me --ignore-time -m x$i --all f
 done
 
 cd ..
