@@ -1,29 +1,29 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
 rm -rf temp1 temp2
 mkdir temp1
 cd temp1
-$DARCS init
+darcs init
 echo record author me > _darcs/prefs/defaults
 echo ALL all >> _darcs/prefs/defaults
 echo ALL verbose >> _darcs/prefs/defaults
 echo ALL ignore-times >> _darcs/prefs/defaults
 touch foo
-$DARCS add foo
-$DARCS record -m add_foo
+darcs add foo
+darcs whatsnew
+darcs record -m add_foo
 echo hello >> foo
-$DARCS record -m mod_foo
+darcs record -m mod_foo
 cd ..
-$DARCS get --repo-name temp2 temp1
+darcs get --repo-name temp2 temp1
 cd temp2
 cp ../temp1/_darcs/prefs/defaults _darcs/prefs
-echo y/d/y | tr / \\012 | $DARCS unpull --interactive
-test -e foo -a ! -s foo
+echo y/d/y | tr / \\012 | darcs unpull --interactive
+test -f foo -a ! -s foo
 echo hello >> foo
-$DARCS record -m mod_foo_again
-$DARCS pull ../temp1
+darcs record -m mod_foo_again
+darcs pull ../temp1
 test -s foo
 cd ..
 rm -rf temp1 temp2
