@@ -26,6 +26,9 @@ rm -rf temp2
 mkdir temp2
 cd temp2
 darcs init
+darcs apply ../temp1/funpatch
+## Also test that "darcs apply" can accept a patch on stdin.
+darcs obl -a
 darcs apply < ../temp1/funpatch
 cd ..
 cmp temp1/bar temp2/bar
