@@ -1,9 +1,6 @@
 #!/usr/bin/env bash
 set -ev
 
-rm -rf test
-mkdir test
-cd test
 darcs init
 echo zig > foo
 darcs add foo
@@ -24,7 +21,3 @@ darcs show authors --verbose
 darcs show authors --verbose | grep y | wc -l > num-patches
 cat num-patches
 grep 2 num-patches
-
-cd ..
-rm -rf test
-
