@@ -3,10 +3,6 @@
 ## All these commands SHOULD fail (hence leading NOTs).
 . lib
 
-rm -rf test
-mkdir test
-cd test
-
 darcs show bug --debug 1> stdout 2> stderr || true
 
 cat stdout
@@ -24,7 +20,3 @@ not grep 'please do not' stderr
 #not grep unable stderr
 
 grep 'fake bug' stderr
-
-cd ..
-rm -rf test
-
