@@ -24,7 +24,7 @@
 ## SOFTWARE.
 
 set -ev
-. ../tests/lib
+. lib
 rm -rf R
 darcs init      --repo R
 touch R/foo
