@@ -2,7 +2,7 @@
 
 . lib
 
-DIR=`hspwd`
+DIR="`pwd`"
 
 abort_windows
 
