@@ -1,25 +1,23 @@
-#!/bin/sh
+#!/usr/bin/env bash
 set -ev
 
-test $DARCS || DARCS=$PWD/../darcs
-
 rm -rf temp1
 mkdir temp1
 cd temp1
-$DARCS init
+darcs init
 cat > f <<EOF
 one
 two
 three
 EOF
-$DARCS rec -Ax -alm init
+darcs rec -Ax -alm init
 cp f g
 cat > f <<EOF
 three
 one
 EOF
-$DARCS rec -Ax -am foo
-echo y | $DARCS unpull -p foo
+darcs rec -Ax -am foo
+echo y | darcs unpull -p foo
 cmp f g
 cd ..
 rm -rf temp1
