@@ -41,6 +41,7 @@ grep_changes () {
     if grep "$1" changes.xml ; then
         echo "$2 OK"
     else
+        cat changes.xml
         echo "$2 not UTF-8-encoded!"
         exit 1
     fi
@@ -70,7 +71,6 @@ echo 'uroh4xx0rz' >> interaction_script.txt
 echo n >> interaction_script.txt
 
 unset DARCSEMAIL
-unset DARCS_TESTING_PREFS_DIR
 unset EMAIL
 set
 darcs record -i < interaction_script.txt
@@ -117,9 +117,13 @@ echo 'No, it is really 484' >> something.txt
 echo y | darcs amend-record -p 'Patch by ' -A 'ed is even deader' -a
 grep_changes 'Žed is even deader' 'author name from amend-record command line flag'
 
-echo '#!/usr/bin/env bash' > editor
-echo 'echo All my s are gone > $1' >> editor # create an 'editor' that writes latin9
-chmod +x editor
+cat <<FAKE > editor.hs # create an 'editor' that writes latin9
+import System.Environment
+import qualified Data.ByteString as B
+str = B.pack [65,108,108,32,109,121,32,164,115,32,97,114,101,32,103,111,110,101]
+main = getArgs >>= \[x] -> B.writeFile x str
+FAKE
+ghc --make -o editor editor.hs
 export EDITOR="`pwd`/editor"
 printf "y\ny\n" | darcs amend --edit -p 'Patch by '
 grep_changes 'All my €s are gone' 'description edited from amend-record'
